package java.util.regex;

import java.util.stream.StreamSupport;
import java.util.Spliterators;
import java.util.NoSuchElementException;
import java.util.Iterator;
import java.util.stream.Stream;
import java.util.function.Predicate;
import java.util.Locale;
import java.util.HashMap;
import java.util.Arrays;
import java.text.Normalizer;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.util.ArrayList;
import java.util.Map;
import java.io.Serializable;

public final class Pattern implements Serializable
{
    public static final int UNIX_LINES = 1;
    public static final int CASE_INSENSITIVE = 2;
    public static final int COMMENTS = 4;
    public static final int MULTILINE = 8;
    public static final int LITERAL = 16;
    public static final int DOTALL = 32;
    public static final int UNICODE_CASE = 64;
    public static final int CANON_EQ = 128;
    public static final int UNICODE_CHARACTER_CLASS = 256;
    private static final long serialVersionUID = 5073258162644648461L;
    private String pattern;
    private int flags;
    private transient volatile boolean compiled;
    private transient String normalizedPattern;
    transient Pattern.Node root;
    transient Pattern.Node matchRoot;
    transient int[] buffer;
    transient volatile Map<String, Integer> namedGroups;
    transient Pattern.GroupHead[] groupNodes;
    private transient int[] temp;
    transient int capturingGroupCount;
    transient int localCount;
    private transient int cursor;
    private transient int patternLength;
    private transient boolean hasSupplementary;
    static final int MAX_REPS = Integer.MAX_VALUE;
    static final int GREEDY = 0;
    static final int LAZY = 1;
    static final int POSSESSIVE = 2;
    static final int INDEPENDENT = 3;
    static Pattern.Node lookbehindEnd;
    static Pattern.Node accept;
    static Pattern.Node lastAccept;
    
    public static Pattern compile(String s) {
        return new Pattern(s, 0);
    }
    
    public static Pattern compile(String s, int n) {
        return new Pattern(s, n);
    }
    
    public String pattern() {
        return this.pattern;
    }
    
    @Override
    public String toString() {
        return this.pattern;
    }
    
    public Matcher matcher(CharSequence charSequence) {
        if (!this.compiled) {
            synchronized (this) {
                if (!this.compiled) {
                    this.compile();
                }
            }
        }
        return new Matcher(this, charSequence);
    }
    
    public int flags() {
        return this.flags;
    }
    
    public static boolean matches(String s, CharSequence charSequence) {
        return compile(s).matcher(charSequence).matches();
    }
    
    public String[] split(CharSequence charSequence, int n) {
        int n2 = 0;
        int n3 = n > 0;
        ArrayList<String> list = new ArrayList();
        Matcher matcher = this.matcher(charSequence);
        while (matcher.find()) {
            if (n3 == 0 || list.size() < n - 1) {
                if (n2 == 0 && n2 == matcher.start() && matcher.start() == matcher.end()) {
                    continue;
                }
                list.add(charSequence.subSequence(n2, matcher.start()).toString());
                n2 = matcher.end();
            }
            else {
                if (list.size() != n - 1) {
                    continue;
                }
                list.add(charSequence.subSequence(n2, charSequence.length()).toString());
                n2 = matcher.end();
            }
        }
        if (n2 == 0) {
            return new String[] { charSequence.toString() };
        }
        if (n3 == 0 || list.size() < n) {
            list.add(charSequence.subSequence(n2, charSequence.length()).toString());
        }
        int size = list.size();
        if (n == 0) {
            while (size > 0 && ((String)list.get(size - 1)).equals("")) {
                --size;
            }
        }
        return (String[])list.subList(0, size).toArray(new String[size]);
    }
    
    public String[] split(CharSequence charSequence) {
        return this.split(charSequence, 0);
    }
    
    public static String quote(String str) {
        if (str.indexOf("\\E") == -1) {
            return "\\Q" + str + "\\E";
        }
        StringBuilder sb = new StringBuilder(str.length() * 2);
        sb.append("\\Q");
        int beginIndex = 0;
        int index;
        while ((index = str.indexOf("\\E", beginIndex)) != -1) {
            sb.append(str.substring(beginIndex, index));
            beginIndex = index + 2;
            sb.append("\\E\\\\E\\Q");
        }
        sb.append(str.substring(beginIndex, str.length()));
        sb.append("\\E");
        return sb.toString();
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws IOException, ClassNotFoundException {
        objectInputStream.defaultReadObject();
        this.capturingGroupCount = 1;
        this.localCount = 0;
        this.compiled = false;
        if (this.pattern.length() == 0) {
            this.root = new Pattern.Start(Pattern.lastAccept);
            this.matchRoot = Pattern.lastAccept;
            this.compiled = true;
        }
    }
    
    private Pattern(String pattern, int flags) {
        this.compiled = false;
        this.pattern = pattern;
        this.flags = flags;
        if ((this.flags & 0x100) != 0x0) {
            this.flags |= 0x40;
        }
        this.capturingGroupCount = 1;
        this.localCount = 0;
        if (this.pattern.length() > 0) {
            try {
                this.compile();
            }
            catch (StackOverflowError stackOverflowError) {
                throw this.error("Stack overflow during pattern compilation");
            }
        }
        else {
            this.root = new Pattern.Start(Pattern.lastAccept);
            this.matchRoot = Pattern.lastAccept;
        }
    }
    
    private void normalize() {
        int n = -1;
        this.normalizedPattern = Normalizer.normalize(this.pattern, Normalizer.Form.NFD);
        this.patternLength = this.normalizedPattern.length();
        StringBuilder sb = new StringBuilder(this.patternLength);
        int codePoint;
        for (int i = 0; i < this.patternLength; n = codePoint, i += Character.charCount(codePoint)) {
            codePoint = this.normalizedPattern.codePointAt(i);
            if (Character.getType(codePoint) == 6 && n != -1) {
                StringBuilder o = new StringBuilder();
                o.appendCodePoint(n);
                o.appendCodePoint(codePoint);
                while (Character.getType(codePoint) == 6) {
                    i += Character.charCount(codePoint);
                    if (i >= this.patternLength) {
                        break;
                    }
                    codePoint = this.normalizedPattern.codePointAt(i);
                    o.appendCodePoint(codePoint);
                }
                String produceEquivalentAlternation = this.produceEquivalentAlternation(o.toString());
                sb.setLength(sb.length() - Character.charCount(n));
                sb.append("(?:").append(produceEquivalentAlternation).append(")");
            }
            else if (codePoint == 91 && n != 92) {
                i = this.normalizeCharClass(sb, i);
            }
            else {
                sb.appendCodePoint(codePoint);
            }
        }
        this.normalizedPattern = sb.toString();
    }
    
    private int normalizeCharClass(StringBuilder sb, int n) {
        StringBuilder sb2 = new StringBuilder();
        Object o = null;
        int n2 = -1;
        if (++n == this.normalizedPattern.length()) {
            throw this.error("Unclosed character class");
        }
        sb2.append("[");
        while (true) {
            int codePoint = this.normalizedPattern.codePointAt(n);
            if (codePoint == 93 && n2 != 92) {
                sb2.append((char)codePoint);
                String str;
                if (o != null) {
                    str = "(?:" + sb2.toString() + o.toString() + ")";
                }
                else {
                    str = sb2.toString();
                }
                sb.append(str);
                return n;
            }
            else {
                if (Character.getType(codePoint) == 6) {
                    StringBuilder o2 = new StringBuilder();
                    o2.appendCodePoint(n2);
                    while (Character.getType(codePoint) == 6) {
                        o2.appendCodePoint(codePoint);
                        n += Character.charCount(codePoint);
                        if (n >= this.normalizedPattern.length()) {
                            break;
                        }
                        codePoint = this.normalizedPattern.codePointAt(n);
                    }
                    String produceEquivalentAlternation = this.produceEquivalentAlternation(o2.toString());
                    sb2.setLength(sb2.length() - Character.charCount(n2));
                    if (o == null) {
                        o = new StringBuilder();
                    }
                    o.append('|');
                    o.append(produceEquivalentAlternation);
                }
                else {
                    sb2.appendCodePoint(codePoint);
                    ++n;
                }
                if (n == this.normalizedPattern.length()) {
                    throw this.error("Unclosed character class");
                }
                n2 = codePoint;
            }
        }
    }
    
    private String produceEquivalentAlternation(String str) {
        int countChars = countChars(str, 0, 1);
        if (str.length() == countChars) {
            return str;
        }
        String substring = str.substring(0, countChars);
        String[] producePermutations = this.producePermutations(str.substring(countChars));
        StringBuilder sb = new StringBuilder(str);
        for (int i = 0; i < producePermutations.length; ++i) {
            String string = substring + producePermutations[i];
            if (i > 0) {
                sb.append("|" + string);
            }
            String composeOneStep = this.composeOneStep(string);
            if (composeOneStep != null) {
                sb.append("|" + this.produceEquivalentAlternation(composeOneStep));
            }
        }
        return sb.toString();
    }
    
    private String[] producePermutations(String s) {
        if (s.length() == countChars(s, 0, 1)) {
            return new String[] { s };
        }
        if (s.length() == countChars(s, 0, 2)) {
            int codePoint2 = Character.codePointAt(s, 0);
            int codePoint3 = Character.codePointAt(s, Character.charCount(codePoint2));
            if (this.getClass(codePoint3) == this.getClass(codePoint2)) {
                return new String[] { s };
            }
            String[] array4 = { s, null };
            StringBuilder sb = new StringBuilder(2);
            sb.appendCodePoint(codePoint3);
            sb.appendCodePoint(codePoint2);
            array4[1] = sb.toString();
            return array4;
        }
        else {
            int n = 1;
            int countCodePoints = countCodePoints(s);
            for (int i = 1; i < countCodePoints; ++i) {
                n *= i + 1;
            }
            String[] array = new String[n];
            int[] array2 = new int[countCodePoints];
            int j = 0;
            int index = 0;
            while (j < countCodePoints) {
                int codePoint = Character.codePointAt(s, index);
                array2[j] = this.getClass(codePoint);
                index += Character.charCount(codePoint);
                ++j;
            }
            int n2 = 0;
            int k = 0;
            int n3 = 0;
        Label_0215:
            while (k < countCodePoints) {
                int countChars = countChars(s, n3, 1);
                int l = k - 1;
                Label_0362: {
                    while (l >= 0) {
                        if (array2[l] == array2[k]) {
                            ++k;
                            n3 += countChars;
                            continue Label_0215;
                        }
                        --l;
                    }
                    String[] producePermutations = this.producePermutations(new StringBuilder(s).delete(n3, n3 + countChars).toString());
                    String substring = s.substring(n3, n3 + countChars);
                    for (int n4 = 0; n4 < producePermutations.length; ++n4) {
                        array[n2++] = substring + producePermutations[n4];
                    }
                    continue;
                }
            }
            String[] array3 = new String[n2];
            for (int n5 = 0; n5 < n2; ++n5) {
                array3[n5] = array[n5];
            }
            return array3;
        }
    }
    
    private int getClass(int ch) {
        return sun.text.Normalizer.getCombiningClass(ch);
    }
    
    private String composeOneStep(String s) {
        int countChars = countChars(s, 0, 2);
        String substring = s.substring(0, countChars);
        String normalize = Normalizer.normalize(substring, Normalizer.Form.NFC);
        if (normalize.equals(substring)) {
            return null;
        }
        return normalize + s.substring(countChars);
    }
    
    private void RemoveQEQuoting() {
        int patternLength = this.patternLength;
        int i = 0;
        while (i < patternLength - 1) {
            if (this.temp[i] != 92) {
                ++i;
            }
            else if (this.temp[i + 1] != 81) {
                i += 2;
            }
            else {
                break;
            }
        }
        if (i >= patternLength - 1) {
            return;
        }
        i += 2;
        int[] original = new int[i + 3 * (patternLength - i) + 2];
        System.arraycopy(this.temp, 0, original, 0, i);
        int n = 1;
        int n2 = 1;
        while (i < patternLength) {
            int n3 = this.temp[i++];
            if (!ASCII.isAscii(n3) || ASCII.isAlpha(n3)) {
                original[i++] = n3;
            }
            else if (ASCII.isDigit(n3)) {
                if (n2 != 0) {
                    original[i++] = 92;
                    original[i++] = 120;
                    original[i++] = 51;
                }
                original[i++] = n3;
            }
            else if (n3 != 92) {
                if (n != 0) {
                    original[i++] = 92;
                }
                original[i++] = n3;
            }
            else if (n != 0) {
                if (this.temp[i] == 69) {
                    ++i;
                    n = 0;
                }
                else {
                    original[i++] = 92;
                    original[i++] = 92;
                }
            }
            else {
                if (this.temp[i] == 81) {
                    ++i;
                    n = 1;
                    n2 = 1;
                    continue;
                }
                original[i++] = n3;
                if (i != patternLength) {
                    original[i++] = this.temp[i++];
                }
            }
            n2 = 0;
        }
        this.patternLength = i;
        this.temp = Arrays.copyOf(original, i + 2);
        return;
    }
    
    private void compile() {
        if (this.has(128) && !this.has(16)) {
            this.normalize();
        }
        else {
            this.normalizedPattern = this.pattern;
        }
        this.patternLength = this.normalizedPattern.length();
        this.temp = new int[this.patternLength + 2];
        this.hasSupplementary = false;
        int patternLength = 0;
        int codePoint;
        for (int i = 0; i < this.patternLength; this.temp[patternLength++] = codePoint, i += Character.charCount(codePoint)) {
            codePoint = this.normalizedPattern.codePointAt(i);
            if (isSupplementary(codePoint)) {
                this.hasSupplementary = true;
            }
        }
        this.patternLength = patternLength;
        if (!this.has(16)) {
            this.RemoveQEQuoting();
        }
        this.buffer = new int[32];
        this.groupNodes = new Pattern.GroupHead[10];
        this.namedGroups = null;
        if (this.has(16)) {
            this.matchRoot = this.newSlice(this.temp, this.patternLength, this.hasSupplementary);
            this.matchRoot.next = Pattern.lastAccept;
        }
        else {
            this.matchRoot = this.expr(Pattern.lastAccept);
            if (this.patternLength != this.cursor) {
                if (this.peek() == 41) {
                    throw this.error("Unmatched closing ')'");
                }
                throw this.error("Unexpected internal error");
            }
        }
        if ((this.matchRoot instanceof Pattern.Slice)) {
            this.root = Pattern.BnM.optimize(this.matchRoot);
            if (this.root == this.matchRoot) {
                this.root = (this.hasSupplementary ? new Pattern.StartS(this.matchRoot) : new Pattern.Start(this.matchRoot));
            }
        }
        else if (this.matchRoot instanceof Pattern.Begin || this.matchRoot instanceof Pattern.First) {
            this.root = this.matchRoot;
        }
        else {
            this.root = (this.hasSupplementary ? new Pattern.StartS(this.matchRoot) : new Pattern.Start(this.matchRoot));
        }
        this.temp = null;
        this.buffer = null;
        this.groupNodes = null;
        this.patternLength = 0;
        this.compiled = true;
    }
    
    Map<String, Integer> namedGroups() {
        if (this.namedGroups == null) {
            this.namedGroups = new HashMap(2);
        }
        return this.namedGroups;
    }
    
    private static void printObjectTree(Pattern.Node next) {
        while (next != null) {
            if ((next instanceof Pattern.Prolog)) {
                System.out.println(next);
                printObjectTree(((Pattern.Prolog)next).loop);
                System.out.println("**** end contents prolog loop");
            }
            else if ((next instanceof Pattern.Loop)) {
                System.out.println(next);
                printObjectTree(((Pattern.Loop)next).body);
                System.out.println("**** end contents Loop body");
            }
            else if ((next instanceof Pattern.Curly)) {
                System.out.println(next);
                printObjectTree(((Pattern.Curly)next).atom);
                System.out.println("**** end contents Curly body");
            }
            else if ((next instanceof Pattern.GroupCurly)) {
                System.out.println(next);
                printObjectTree(((Pattern.GroupCurly)next).atom);
                System.out.println("**** end contents GroupCurly body");
            }
            else {
                if ((next instanceof Pattern.GroupTail)) {
                    System.out.println(next);
                    System.out.println("Tail next is " + next.next);
                    return;
                }
                System.out.println(next);
            }
            next = next.next;
            if (next != null) {
                System.out.println("->next:");
            }
            if (next != Pattern.accept) {
                continue;
            }
            System.out.println("Accept Node");
            next = null;
        }
    }
    
    private boolean has(int n) {
        return (this.flags & n) != 0x0;
    }
    
    private void accept(int n, String s) {
        int pastWhitespace = this.temp[this.cursor++];
        if (this.has(4)) {
            pastWhitespace = this.parsePastWhitespace(pastWhitespace);
        }
        if (n != pastWhitespace) {
            throw this.error(s);
        }
        return;
    }
    
    private void mark(int n) {
        this.temp[this.patternLength] = n;
    }
    
    private int peek() {
        int peekPastWhitespace = this.temp[this.cursor];
        if (this.has(4)) {
            peekPastWhitespace = this.peekPastWhitespace(peekPastWhitespace);
        }
        return peekPastWhitespace;
    }
    
    private int read() {
        int pastWhitespace = this.temp[this.cursor++];
        if (this.has(4)) {
            pastWhitespace = this.parsePastWhitespace(pastWhitespace);
        }
        return pastWhitespace;
    }
    
    private int readEscaped() {
        return this.temp[this.cursor++];
    }
    
    private int next() {
        int peekPastWhitespace = this.temp[++this.cursor];
        if (this.has(4)) {
            peekPastWhitespace = this.peekPastWhitespace(peekPastWhitespace);
        }
        return peekPastWhitespace;
    }
    
    private int nextEscaped() {
        return this.temp[++this.cursor];
    }
    
    private int peekPastWhitespace(int peekPastLine) {
        while (ASCII.isSpace(peekPastLine) || peekPastLine == 35) {
            while (ASCII.isSpace(peekPastLine)) {
                peekPastLine = this.temp[++this.cursor];
            }
            if (peekPastLine != 35) {
                continue;
            }
            peekPastLine = this.peekPastLine();
        }
        return peekPastLine;
    }
    
    private int parsePastWhitespace(int pastLine) {
        while (ASCII.isSpace(pastLine) || pastLine == 35) {
            while (ASCII.isSpace(pastLine)) {
                pastLine = this.temp[this.cursor++];
            }
            if (pastLine != 35) {
                continue;
            }
            pastLine = this.parsePastLine();
        }
        return pastLine;
    }
    
    private int parsePastLine() {
        int n;
        for (n = this.temp[this.cursor++]; n != 0 && !this.isLineSeparator(n); n = this.temp[this.cursor++]) {}
        if (n == 0 && this.cursor > this.patternLength) {
            this.cursor = this.patternLength;
            n = this.temp[this.cursor++];
        }
        return n;
    }
    
    private int peekPastLine() {
        int n;
        for (n = this.temp[++this.cursor]; n != 0 && !this.isLineSeparator(n); n = this.temp[++this.cursor]) {}
        if (n == 0 && this.cursor > this.patternLength) {
            this.cursor = this.patternLength;
            n = this.temp[this.cursor];
        }
        return n;
    }
    
    private boolean isLineSeparator(int n) {
        if (this.has(1)) {
            return n == 10;
        }
        return n == 10 || n == 13 || (n | 0x1) == 0x2029 || n == 133;
    }
    
    private int skip() {
        int cursor = this.cursor;
        int n = this.temp[cursor + 1];
        this.cursor = cursor + 2;
        return n;
    }
    
    private void unread() {
        --this.cursor;
    }
    
    private PatternSyntaxException error(String s) {
        return new PatternSyntaxException(s, this.normalizedPattern, this.cursor - 1);
    }
    
    private boolean findSupplementary(int i, int n) {
        while (i < n) {
            if (isSupplementary(this.temp[i])) {
                return true;
            }
            ++i;
        }
        return false;
    }
    
    private static final boolean isSupplementary(int n) {
        return n >= 65536 || Character.isSurrogate((char)n);
    }
    
    private Pattern.Node expr(Pattern.Node next) {
        Object o = null;
        Pattern.Node node = null;
        Object o2 = null;
        Object o3 = null;
        while (true) {
            Pattern.Node sequence = this.sequence(next);
            Pattern.Node root = this.root;
            if (o == null) {
                o = sequence;
                node = root;
            }
            else {
                if (o3 == null) {
                    o3 = new Pattern.BranchConn();
                    o3.next = next;
                }
                if (sequence == next) {
                    sequence = null;
                }
                else {
                    root.next = o3;
                }
                if (o == o2) {
                    o2.add(sequence);
                }
                else {
                    if (o == next) {
                        o = null;
                    }
                    else {
                        node.next = o3;
                    }
                    o2 = (o = new Pattern.Branch(o, sequence, o3));
                }
            }
            if (this.peek() != 124) {
                break;
            }
            this.next();
        }
        return o;
    }
    
    private Pattern.Node sequence(Pattern.Node next) {
        Pattern.Node closure;
    Label_0535:
        while (true) {
            int peek = this.peek();
            Pattern.CharProperty charProperty;
            Label_0503: {
                switch (peek) {
                    case 40: {
                        Pattern.Node group0 = this.group0();
                        if (group0 == null) {
                            continue;
                        }
                        if (closure != null) {
                            closure.next = group0;
                        }
                        Pattern.Node root = this.root;
                        continue;
                    }
                    case 91: {
                        charProperty = this.clazz(true);
                        break Label_0503;
                    }
                    case 92: {
                        int nextEscaped = this.nextEscaped();
                        if (nextEscaped == 112 || nextEscaped == 80) {
                            boolean n = true;
                            int n2 = nextEscaped == 80;
                            if (this.next() != 123) {
                                this.unread();
                            }
                            else {
                                n = false;
                            }
                            charProperty = this.family((boolean)n, (boolean)n2);
                            break Label_0503;
                        }
                        else {
                            this.unread();
                            charProperty = this.atom();
                            break Label_0503;
                        }
                    }
                    case 94: {
                        this.next();
                        if (this.has(8)) {
                            if (this.has(1)) {
                                charProperty = new Pattern.UnixCaret();
                                break Label_0503;
                            }
                            charProperty = new Pattern.Caret();
                            break Label_0503;
                        }
                        else {
                            charProperty = new Pattern.Begin();
                            break Label_0503;
                        }
                    }
                    case 36: {
                        this.next();
                        if (this.has(1)) {
                            charProperty = new Pattern.UnixDollar(this.has(8));
                            break Label_0503;
                        }
                        charProperty = new Pattern.Dollar(this.has(8));
                        break Label_0503;
                    }
                    case 46: {
                        this.next();
                        if (this.has(32)) {
                            charProperty = new Pattern.All();
                            break Label_0503;
                        }
                        if (this.has(1)) {
                            charProperty = new Pattern.UnixDot();
                            break Label_0503;
                        }
                        charProperty = new Pattern.Dot();
                        break Label_0503;
                    }
                    case 41:
                    case 124: {
                        break Label_0535;
                    }
                    case 93:
                    case 125: {
                        charProperty = this.atom();
                        break Label_0503;
                    }
                    case 42:
                    case 43:
                    case 63: {
                        this.next();
                        throw this.error("Dangling meta character '" + (char)peek + "'");
                    }
                    case 0: {
                        if (this.cursor >= this.patternLength) {
                            break Label_0535;
                        }
                        break;
                    }
                }
                charProperty = this.atom();
            }
            closure = this.closure(charProperty);
            if (closure == null) {
                continue;
            }
            closure.next = closure;
        }
        if (closure == null) {
            return next;
        }
        closure.next = next;
        return this.root = closure;
    }
    
    private Pattern.Node atom() {
        int n = 0;
        int n2 = -1;
        boolean n3 = false;
        int n4 = this.peek();
    Label_0352:
        while (true) {
            switch (n4) {
                case 42:
                case 43:
                case 63:
                case 123: {
                    if (n > 1) {
                        this.cursor = n2;
                        --n;
                    }
                }
                case 36:
                case 40:
                case 41:
                case 46:
                case 91:
                case 94:
                case 124: {
                    break Label_0352;
                }
                case 92: {
                    int nextEscaped = this.nextEscaped();
                    if (nextEscaped == 112 || nextEscaped == 80) {
                        if (n > 0) {
                            this.unread();
                            break Label_0352;
                        }
                        int n5 = nextEscaped == 80;
                        boolean n6 = true;
                        if (this.next() != 123) {
                            this.unread();
                        }
                        else {
                            n6 = false;
                        }
                        return this.family((boolean)n6, (boolean)n5);
                    }
                    else {
                        this.unread();
                        n2 = this.cursor;
                        int escape = this.escape(false, n == 0, false);
                        if (escape >= 0) {
                            this.append(escape, n);
                            ++n;
                            if (isSupplementary(escape)) {
                                n3 = true;
                            }
                            n4 = this.peek();
                            continue;
                        }
                        else {
                            if (n == 0) {
                                return this.root;
                            }
                            this.cursor = n2;
                            break Label_0352;
                        }
                    }
                }
                case 0: {
                    if (this.cursor >= this.patternLength) {
                        break Label_0352;
                    }
                    break;
                }
            }
            n2 = this.cursor;
            this.append(n4, n);
            ++n;
            if (isSupplementary(n4)) {
                n3 = true;
            }
            n4 = this.next();
        }
        if (n == 1) {
            return this.newSingle(this.buffer[0]);
        }
        return this.newSlice(this.buffer, n, (boolean)n3);
    }
    
    private void append(int n, int n2) {
        if (n2 >= this.buffer.length) {
            int[] buffer = new int[n2 + n2];
            System.arraycopy(this.buffer, 0, buffer, 0, n2);
            this.buffer = buffer;
        }
        this.buffer[n2] = n;
    }
    
    private Pattern.Node ref(int n) {
        int i = 0;
        while (i == 0) {
            int peek = this.peek();
            switch (peek) {
                case 48:
                case 49:
                case 50:
                case 51:
                case 52:
                case 53:
                case 54:
                case 55:
                case 56:
                case 57: {
                    int n2 = n * 10 + (peek - 48);
                    if (this.capturingGroupCount - 1 < n2) {
                        i = 1;
                        continue;
                    }
                    n = n2;
                    this.read();
                    continue;
                }
                default: {
                    i = 1;
                    continue;
                }
            }
        }
        if (this.has(2)) {
            return new Pattern.CIBackRef(n, this.has(64));
        }
        return new BackRef(n);
    }
    
    private int escape(boolean b, boolean b2, boolean b3) {
        int skip = this.skip();
        switch (skip) {
            case 48: {
                return this.o();
            }
            case 49:
            case 50:
            case 51:
            case 52:
            case 53:
            case 54:
            case 55:
            case 56:
            case 57: {
                if (b) {
                    break;
                }
                if (b2) {
                    this.root = this.ref(skip - 48);
                }
                return -1;
            }
            case 65: {
                if (b) {
                    break;
                }
                if (b2) {
                    this.root = new Pattern.Begin();
                }
                return -1;
            }
            case 66: {
                if (!b) {
                    if (b2) {
                        this.root = new Pattern.Bound(Pattern.Bound.NONE, this.has(256));
                    }
                    return -1;
                }
            }
            case 67: {
                break;
            }
            case 68: {
                if (b2) {
                    this.root = (this.has(256) ? new Pattern.Utype(UnicodeProp.DIGIT).complement() : new Pattern.Ctype(1024).complement());
                }
                return -1;
            }
            case 69:
            case 70: {
                break;
            }
            case 71: {
                if (b) {
                    break;
                }
                if (b2) {
                    this.root = new Pattern.LastMatch();
                }
                return -1;
            }
            case 72: {
                if (b2) {
                    this.root = new Pattern.HorizWS().complement();
                }
                return -1;
            }
            case 73:
            case 74:
            case 75:
            case 76:
            case 77:
            case 78:
            case 79:
            case 80:
            case 81: {
                break;
            }
            case 82: {
                if (b) {
                    break;
                }
                if (b2) {
                    this.root = new Pattern.LineEnding();
                }
                return -1;
            }
            case 83: {
                if (b2) {
                    this.root = (this.has(256) ? new Pattern.Utype(UnicodeProp.WHITE_SPACE).complement() : new Pattern.Ctype(2048).complement());
                }
                return -1;
            }
            case 84:
            case 85: {
                break;
            }
            case 86: {
                if (b2) {
                    this.root = new Pattern.VertWS().complement();
                }
                return -1;
            }
            case 87: {
                if (b2) {
                    this.root = (this.has(256) ? new Pattern.Utype(UnicodeProp.WORD).complement() : new Pattern.Ctype(67328).complement());
                }
                return -1;
            }
            case 88:
            case 89: {
                break;
            }
            case 90: {
                if (b) {
                    break;
                }
                if (b2) {
                    if (this.has(1)) {
                        this.root = new Pattern.UnixDollar(false);
                    }
                    else {
                        this.root = new Pattern.Dollar(false);
                    }
                }
                return -1;
            }
            case 97: {
                return 7;
            }
            case 98: {
                if (b) {
                    break;
                }
                if (b2) {
                    this.root = new Pattern.Bound(Pattern.Bound.BOTH, this.has(256));
                }
                return -1;
            }
            case 99: {
                return this.c();
            }
            case 100: {
                if (b2) {
                    this.root = (this.has(256) ? new Pattern.Utype(UnicodeProp.DIGIT) : new Pattern.Ctype(1024));
                }
                return -1;
            }
            case 101: {
                return 27;
            }
            case 102: {
                return 12;
            }
            case 103: {
                break;
            }
            case 104: {
                if (b2) {
                    this.root = new Pattern.HorizWS();
                }
                return -1;
            }
            case 105:
            case 106: {
                break;
            }
            case 107: {
                if (!b) {
                    if (this.read() != 60) {
                        throw this.error("\\k is not followed by '<' for named capturing group");
                    }
                    String groupname = this.groupname(this.read());
                    if (!this.namedGroups().containsKey(groupname)) {
                        throw this.error("(named capturing group <" + groupname + "> does not exit");
                    }
                    if (b2) {
                        if (this.has(2)) {
                            this.root = new Pattern.CIBackRef((int)this.namedGroups().get(groupname), this.has(64));
                        }
                        else {
                            this.root = new Pattern.BackRef((int)this.namedGroups().get(groupname));
                        }
                    }
                    return -1;
                }
            }
            case 108:
            case 109: {
                break;
            }
            case 110: {
                return 10;
            }
            case 111:
            case 112:
            case 113: {
                break;
            }
            case 114: {
                return 13;
            }
            case 115: {
                if (b2) {
                    this.root = (this.has(256) ? new Pattern.Utype(UnicodeProp.WHITE_SPACE) : new Pattern.Ctype(2048));
                }
                return -1;
            }
            case 116: {
                return 9;
            }
            case 117: {
                return this.u();
            }
            case 118: {
                if (b3) {
                    return 11;
                }
                if (b2) {
                    this.root = new Pattern.VertWS();
                }
                return -1;
            }
            case 119: {
                if (b2) {
                    this.root = (this.has(256) ? new Pattern.Utype(UnicodeProp.WORD) : new Pattern.Ctype(67328));
                }
                return -1;
            }
            case 120: {
                return this.x();
            }
            case 121: {
                break;
            }
            case 122: {
                if (b) {
                    break;
                }
                if (b2) {
                    this.root = new Pattern.End();
                }
                return -1;
            }
            default: {
                return skip;
            }
        }
        throw this.error("Illegal/unsupported escape sequence");
    }
    
    private Pattern.CharProperty clazz(boolean b) {
        Pattern.CharProperty charProperty = null;
        Pattern.CharProperty charProperty2 = null;
        Pattern.BitClass bitClass = new Pattern.BitClass();
        boolean n = true;
        int n2 = 1;
        int n3 = this.next();
        while (true) {
            switch (n3) {
                case 94: {
                    if (n2 != 0) {
                        if (this.temp[this.cursor - 1] != 91) {
                            break;
                        }
                        n3 = this.next();
                        n = (n == 0);
                        continue;
                    }
                    else {
                        break;
                    }
                }
                case 91: {
                    n2 = 0;
                    charProperty2 = this.clazz(true);
                    if (charProperty == null) {
                        charProperty = charProperty2;
                    }
                    else {
                        charProperty = union(charProperty, charProperty2);
                    }
                    n3 = this.peek();
                    continue;
                }
                case 38: {
                    n2 = 0;
                    if (this.next() == 38) {
                        n3 = this.next();
                        Pattern.CharProperty charProperty3 = null;
                        while (n3 != 93 && n3 != 38) {
                            if (n3 == 91) {
                                if (charProperty3 == null) {
                                    charProperty3 = this.clazz(true);
                                }
                                else {
                                    charProperty3 = union(charProperty3, this.clazz(true));
                                }
                            }
                            else {
                                this.unread();
                                charProperty3 = this.clazz(false);
                            }
                            n3 = this.peek();
                        }
                        if (charProperty3 != null) {
                            charProperty2 = charProperty3;
                        }
                        if (charProperty == null) {
                            if (charProperty3 == null) {
                                throw this.error("Bad class syntax");
                            }
                            charProperty = charProperty3;
                        }
                        else {
                            charProperty = intersection(charProperty, charProperty2);
                        }
                        continue;
                    }
                    else {
                        this.unread();
                        break;
                    }
                }
                case 0: {
                    n2 = 0;
                    if (this.cursor >= this.patternLength) {
                        throw this.error("Unclosed character class");
                    }
                    break;
                }
                case 93: {
                    n2 = 0;
                    if (charProperty != null) {
                        if (b) {
                            this.next();
                            return charProperty;
                        }
                        return charProperty;
                    }
                    else {
                        break;
                    }
                }
                default: {
                    n2 = 0;
                    break;
                }
            }
            charProperty2 = this.range(bitClass);
            if (n != 0) {
                if (charProperty == null) {
                    charProperty = charProperty2;
                }
                else if (charProperty != charProperty2) {
                    charProperty = union(charProperty, charProperty2);
                }
            }
            else if (charProperty == null) {
                charProperty = charProperty2.complement();
            }
            else if (charProperty != charProperty2) {
                charProperty = setDifference(charProperty, charProperty2);
            }
            n3 = this.peek();
        }
    }
    
    private Pattern.CharProperty bitsOrSingle(Pattern.BitClass bitClass, int n) {
        if (n < 256 && (!this.has(2) || !this.has(64) || (n != 255 && n != 181 && n != 73 && n != 105 && n != 83 && n != 115 && n != 75 && n != 107 && n != 197 && n != 229))) {
            return bitClass.add(n, this.flags());
        }
        return this.newSingle(n);
    }
    
    private Pattern.CharProperty range(Pattern.BitClass bitClass) {
        int n = this.peek();
        if (n == 92) {
            int nextEscaped = this.nextEscaped();
            if (nextEscaped == 112 || nextEscaped == 80) {
                int n3 = nextEscaped == 80;
                boolean n4 = true;
                if (this.next() != 123) {
                    this.unread();
                }
                else {
                    n4 = false;
                }
                return this.family((boolean)n4, (boolean)n3);
            }
            else {
                int n2 = this.temp[this.cursor + 1] == 45;
                this.unread();
                n = this.escape(true, true, (boolean)n2);
                if (n == -1) {
                    return (Pattern.CharProperty)this.root;
                }
            }
        }
        else {
            this.next();
        }
        if (n >= 0) {
            if (this.peek() == 45) {
                int n5 = this.temp[this.cursor + 1];
                if (n5 == 91) {
                    return this.bitsOrSingle(bitClass, n);
                }
                if (n5 != 93) {
                    this.next();
                    int n6 = this.peek();
                    if (n6 == 92) {
                        n6 = this.escape(true, false, true);
                    }
                    else {
                        this.next();
                    }
                    if (n6 < n) {
                        throw this.error("Illegal character range");
                    }
                    if (this.has(2)) {
                        return this.caseInsensitiveRangeFor(n, n6);
                    }
                    return rangeFor(n, n6);
                }
            }
            return this.bitsOrSingle(bitClass, n);
        }
        else {
            throw this.error("Unexpected character '" + (char)n + "'");
        }
    }
    
    private Pattern.CharProperty family(boolean b, boolean b2) {
        this.next();
        Pattern.CharProperty charProperty = null;
        String value;
        if (b) {
            int codePoint = this.temp[this.cursor];
            if (!Character.isSupplementaryCodePoint(codePoint)) {
                value = String.valueOf((char)codePoint);
            }
            else {
                value = new String(this.temp, this.cursor, 1);
            }
            this.read();
        }
        else {
            int cursor = this.cursor;
            this.mark(125);
            while (this.read() != 125) {}
            this.mark(0);
            int cursor2 = this.cursor;
            if (cursor2 > this.patternLength) {
                throw this.error("Unclosed character family");
            }
            if (cursor + 1 >= cursor2) {
                throw this.error("Empty character family");
            }
            value = new String(this.temp, cursor, cursor2 - cursor - 1);
        }
        int index = value.indexOf(61);
        if (index != -1) {
            String substring2 = value.substring(index + 1);
            String lowerCase = value.substring(0, index).toLowerCase(Locale.ENGLISH);
            if ("sc".equals(lowerCase) || "script".equals(lowerCase)) {
                charProperty = this.unicodeScriptPropertyFor(substring2);
            }
            else if ("blk".equals(lowerCase) || "block".equals(lowerCase)) {
                charProperty = this.unicodeBlockPropertyFor(substring2);
            }
            else if ("gc".equals(lowerCase) || "general_category".equals(lowerCase)) {
                charProperty = this.charPropertyNodeFor(substring2);
            }
            else {
                throw this.error("Unknown Unicode property {name=<" + lowerCase + ">, value=<" + substring2 + ">}");
            }
        }
        else if (value.startsWith("In")) {
            charProperty = this.unicodeBlockPropertyFor(value.substring(2));
        }
        else if (value.startsWith("Is")) {
            String substring = value.substring(2);
            UnicodeProp forName = UnicodeProp.forName(substring);
            if (forName != null) {
                charProperty = new Utype(forName);
            }
            if (charProperty == null) {
                charProperty = Pattern.CharPropertyNames.charPropertyFor(substring);
            }
            if (charProperty == null) {
                charProperty = this.unicodeScriptPropertyFor(substring);
            }
        }
        else {
            if (this.has(256)) {
                UnicodeProp forPOSIXName = UnicodeProp.forPOSIXName(value);
                if (forPOSIXName != null) {
                    charProperty = new Utype(forPOSIXName);
                }
            }
            if (charProperty == null) {
                charProperty = this.charPropertyNodeFor(value);
            }
        }
        if (b2) {
            if (charProperty instanceof Pattern.Category || charProperty instanceof Pattern.Block) {
                this.hasSupplementary = true;
            }
            charProperty = charProperty.complement();
            return charProperty;
        }
        else {
            return charProperty;
        }
    }
    
    private Pattern.CharProperty unicodeScriptPropertyFor(String s) {
        Character.UnicodeScript forName;
        try {
            forName = Character.UnicodeScript.forName(s);
        }
        catch (IllegalArgumentException ex) {
            throw this.error("Unknown character script name {" + s + "}");
        }
        return new Script(forName);
    }
    
    private Pattern.CharProperty unicodeBlockPropertyFor(String s) {
        Character.UnicodeBlock forName;
        try {
            forName = Character.UnicodeBlock.forName(s);
        }
        catch (IllegalArgumentException ex) {
            throw this.error("Unknown character block name {" + s + "}");
        }
        return new Block(forName);
    }
    
    private Pattern.CharProperty charPropertyNodeFor(String str) {
        Pattern.CharProperty charProperty = Pattern.CharPropertyNames.charPropertyFor(str);
        if (charProperty == null) {
            throw this.error("Unknown character property name {" + str + "}");
        }
        return charProperty;
    }
    
    private String groupname(int read) {
        StringBuilder sb = new StringBuilder();
        sb.append(Character.toChars(read));
        while (ASCII.isLower(read = this.read()) || ASCII.isUpper(read) || ASCII.isDigit(read)) {
            sb.append(Character.toChars(read));
        }
        if (sb.length() == 0) {
            throw this.error("named capturing group has 0 length name");
        }
        if (read != 62) {
            throw this.error("named capturing group is missing trailing '>'");
        }
        return sb.toString();
    }
    
    private Pattern.Node group0() {
        boolean n = false;
        int flags = this.flags;
        this.root = null;
        Pattern.Node body;
        Pattern.Node root;
        if (this.next() == 63) {
            int skip = this.skip();
            switch (skip) {
                case 58: {
                    body = this.createGroup(true);
                    root = this.root;
                    body.next = this.expr(root);
                    break;
                }
                case 33:
                case 61: {
                    Pattern.Node group = this.createGroup(true);
                    group.next = this.expr(this.root);
                    if (skip == 61) {
                        root = (body = new Pos(group));
                        break;
                    }
                    root = (body = new Neg(group));
                    break;
                }
                case 62: {
                    Pattern.Node group2 = this.createGroup(true);
                    group2.next = this.expr(this.root);
                    root = (body = new Pattern.Ques(group2, 3));
                    break;
                }
                case 60: {
                    int read = this.read();
                    if (ASCII.isLower(read) || ASCII.isUpper(read)) {
                        String groupname = this.groupname(read);
                        if (this.namedGroups().containsKey(groupname)) {
                            throw this.error("Named capturing group <" + groupname + "> is already defined");
                        }
                        n = true;
                        body = this.createGroup(false);
                        root = this.root;
                        this.namedGroups().put(groupname, Integer.valueOf(this.capturingGroupCount - 1));
                        body.next = this.expr(root);
                        break;
                    }
                    else {
                        int cursor = this.cursor;
                        Pattern.Node group3 = this.createGroup(true);
                        Pattern.Node root2 = this.root;
                        group3.next = this.expr(root2);
                        root2.next = Pattern.lookbehindEnd;
                        Pattern.TreeInfo treeInfo = new Pattern.TreeInfo();
                        group3.study(treeInfo);
                        if (!treeInfo.maxValid) {
                            throw this.error("Look-behind group does not have an obvious maximum length");
                        }
                        int supplementary = this.findSupplementary(cursor, this.patternLength);
                        if (read == 61) {
                            root = (body = (supplementary ? new Pattern.BehindS(group3, treeInfo.maxLength, treeInfo.minLength) : new Pattern.Behind(group3, treeInfo.maxLength, treeInfo.minLength)));
                            break;
                        }
                        if (read == 33) {
                            root = (body = (supplementary ? new Pattern.NotBehindS(group3, treeInfo.maxLength, treeInfo.minLength) : new Pattern.NotBehind(group3, treeInfo.maxLength, treeInfo.minLength)));
                            break;
                        }
                        throw this.error("Unknown look-behind group");
                    }
                }
                case 36:
                case 64: {
                    throw this.error("Unknown group type");
                }
                default: {
                    this.unread();
                    this.addFlag();
                    int read2 = this.read();
                    if (read2 == 41) {
                        return null;
                    }
                    if (read2 != 58) {
                        throw this.error("Unknown inline modifier");
                    }
                    body = this.createGroup(true);
                    root = this.root;
                    body.next = this.expr(root);
                    break;
                }
            }
        }
        else {
            n = true;
            body = this.createGroup(false);
            root = this.root;
            body.next = this.expr(root);
        }
        this.accept(41, "Unclosed group");
        this.flags = flags;
        Pattern.Node closure = this.closure(body);
        if (closure == body) {
            this.root = root;
            return closure;
        }
        if (body == root) {
            return this.root = closure;
        }
        if ((closure instanceof Pattern.Ques)) {
            Pattern.Ques ques = (Pattern.Ques)closure;
            if (ques.type == 2) {
                return this.root = closure;
            }
            root.next = new Pattern.BranchConn();
            Pattern.Node next = root.next;
            Object o3;
            if (ques.type == 0) {
                o3 = new Branch(body, null, next);
            }
            else {
                o3 = new Branch(null, body, next);
            }
            this.root = next;
            return o3;
        }
        else if ((closure instanceof Pattern.Curly)) {
            Pattern.Curly curly = (Pattern.Curly)closure;
            if (curly.type == 2) {
                return this.root = closure;
            }
            if (body.study(new Pattern.TreeInfo())) {
                Pattern.GroupTail groupTail = (Pattern.GroupTail)root;
                return this.root = new Pattern.GroupCurly(body.next, curly.cmin, curly.cmax, curly.type, ((Pattern.GroupTail)root).localIndex, ((Pattern.GroupTail)root).groupIndex, (boolean)n);
            }
            int localIndex = ((Pattern.GroupHead)body).localIndex;
            Object o;
            if (curly.type == 0) {
                o = new Loop(this.localCount, localIndex);
            }
            else {
                o = new LazyLoop(this.localCount, localIndex);
            }
            Pattern.Prolog o2 = new Pattern.Prolog(o);
            ++this.localCount;
            o.cmin = curly.cmin;
            o.cmax = curly.cmax;
            o.body = body;
            root.next = o;
            this.root = o;
            return o2;
        }
        else {
            throw this.error("Internal logic error");
        }
    }
    
    private Pattern.Node createGroup(boolean b) {
        int n = this.localCount++;
        int n2 = 0;
        if (!b) {
            n2 = this.capturingGroupCount++;
        }
        Pattern.GroupHead o = new GroupHead(n);
        this.root = new GroupTail(n, n2);
        if (!b && n2 < 10) {
            this.groupNodes[n2] = o;
            return o;
        }
        return o;
    }
    
    private void addFlag() {
        int n = this.peek();
    Label_0208:
        while (true) {
            switch (n) {
                case 105: {
                    this.flags |= 0x2;
                    break;
                }
                case 109: {
                    this.flags |= 0x8;
                    break;
                }
                case 115: {
                    this.flags |= 0x20;
                    break;
                }
                case 100: {
                    this.flags |= 0x1;
                    break;
                }
                case 117: {
                    this.flags |= 0x40;
                    break;
                }
                case 99: {
                    this.flags |= 0x80;
                    break;
                }
                case 120: {
                    this.flags |= 0x4;
                    break;
                }
                case 85: {
                    this.flags |= 0x140;
                    break;
                }
                case 45: {
                    this.next();
                    this.subFlag();
                    break Label_0208;
                }
                default: {
                    break Label_0208;
                }
            }
            n = this.next();
        }
    }
    
    private void subFlag() {
        int n = this.peek();
    Label_0191:
        while (true) {
            switch (n) {
                case 105: {
                    this.flags &= 0xFFFFFFFD;
                    break;
                }
                case 109: {
                    this.flags &= 0xFFFFFFF7;
                    break;
                }
                case 115: {
                    this.flags &= 0xFFFFFFDF;
                    break;
                }
                case 100: {
                    this.flags &= 0xFFFFFFFE;
                    break;
                }
                case 117: {
                    this.flags &= 0xFFFFFFBF;
                    break;
                }
                case 99: {
                    this.flags &= 0xFFFFFF7F;
                    break;
                }
                case 120: {
                    this.flags &= 0xFFFFFFFB;
                    break;
                }
                case 85: {
                    this.flags &= 0xFFFFFEBF;
                    break Label_0191;
                }
                default: {
                    break Label_0191;
                }
            }
            n = this.next();
        }
    }
    
    private Pattern.Node closure(Pattern.Node node) {
        switch (this.peek()) {
            case 63: {
                int next = this.next();
                if (next == 63) {
                    this.next();
                    return new Pattern.Ques(node, 1);
                }
                if (next == 43) {
                    this.next();
                    return new Pattern.Ques(node, 2);
                }
                return new Pattern.Ques(node, 0);
            }
            case 42: {
                int next2 = this.next();
                if (next2 == 63) {
                    this.next();
                    return new Pattern.Curly(node, 0, Integer.MAX_VALUE, 1);
                }
                if (next2 == 43) {
                    this.next();
                    return new Pattern.Curly(node, 0, Integer.MAX_VALUE, 2);
                }
                return new Pattern.Curly(node, 0, Integer.MAX_VALUE, 0);
            }
            case 43: {
                int next3 = this.next();
                if (next3 == 63) {
                    this.next();
                    return new Pattern.Curly(node, 1, Integer.MAX_VALUE, 1);
                }
                if (next3 == 43) {
                    this.next();
                    return new Pattern.Curly(node, 1, Integer.MAX_VALUE, 2);
                }
                return new Pattern.Curly(node, 1, Integer.MAX_VALUE, 0);
            }
            case 123: {
                int n = this.temp[this.cursor + 1];
                if (ASCII.isDigit(n)) {
                    this.skip();
                    int n2 = 0;
                    do {
                        n2 = n2 * 10 + (n - 48);
                    } while (ASCII.isDigit(n = this.read()));
                    if (n == 44) {
                        n = this.read();
                        n2 = Integer.MAX_VALUE;
                        if (n != 125) {
                            n2 = 0;
                            while (ASCII.isDigit(n)) {
                                n2 = n2 * 10 + (n - 48);
                                n = this.read();
                            }
                        }
                    }
                    if (n != 125) {
                        throw this.error("Unclosed counted closure");
                    }
                    if ((n2 | n2 | n2 - n2) < 0) {
                        throw this.error("Illegal repetition range");
                    }
                    int peek = this.peek();
                    Object o;
                    if (peek == 63) {
                        this.next();
                        o = new Pattern.Curly(node, n2, n2, 1);
                    }
                    else if (peek == 43) {
                        this.next();
                        o = new Pattern.Curly(node, n2, n2, 2);
                    }
                    else {
                        o = new Pattern.Curly(node, n2, n2, 0);
                    }
                    return o;
                }
                else {
                    throw this.error("Illegal repetition");
                }
            }
            default: {
                return node;
            }
        }
    }
    
    private int c() {
        if (this.cursor < this.patternLength) {
            return this.read() ^ 0x40;
        }
        throw this.error("Illegal control escape sequence");
    }
    
    private int o() {
        int read = this.read();
        if ((read - 48 | 55 - read) >= 0) {
            int read2 = this.read();
            if ((read2 - 48 | 55 - read2) >= 0) {
                int read3 = this.read();
                if ((read3 - 48 | 55 - read3) >= 0 && (read - 48 | 51 - read) >= 0) {
                    return (read - 48) * 64 + (read2 - 48) * 8 + (read3 - 48);
                }
                this.unread();
                return (read - 48) * 8 + (read2 - 48);
            }
            else {
                this.unread();
                return read - 48;
            }
        }
        else {
            throw this.error("Illegal octal escape sequence");
        }
    }
    
    private int x() {
        int read = this.read();
        if (ASCII.isHexDigit(read)) {
            int read3 = this.read();
            if (ASCII.isHexDigit(read3)) {
                return ASCII.toDigit(read) * 16 + ASCII.toDigit(read3);
            }
        }
        else if (read == 123 && ASCII.isHexDigit(this.peek())) {
            int n = 0;
            int read2;
            while (ASCII.isHexDigit(read2 = this.read())) {
                n = (n << 4) + ASCII.toDigit(read2);
                if (n <= 1114111) {
                    continue;
                }
                throw this.error("Hexadecimal codepoint is too big");
            }
            if (read2 != 125) {
                throw this.error("Unclosed hexadecimal escape sequence");
            }
            return n;
        }
        throw this.error("Illegal hexadecimal escape sequence");
    }
    
    private int cursor() {
        return this.cursor;
    }
    
    private void setcursor(int cursor) {
        this.cursor = cursor;
    }
    
    private int uxxxx() {
        int n = 0;
        int i = 0;
        while (i < 4) {
            int read = this.read();
            if (!ASCII.isHexDigit(read)) {
                throw this.error("Illegal Unicode escape sequence");
            }
            n = n * 16 + ASCII.toDigit(read);
            ++i;
        }
        return n;
    }
    
    private int u() {
        int uxxxx = this.uxxxx();
        if (Character.isHighSurrogate((char)uxxxx)) {
            int cursor = this.cursor();
            if (this.read() == 92 && this.read() == 117) {
                int uxxxx2 = this.uxxxx();
                if (Character.isLowSurrogate((char)uxxxx2)) {
                    return Character.toCodePoint((char)uxxxx, (char)uxxxx2);
                }
            }
            this.setcursor(cursor);
        }
        return uxxxx;
    }
    
    private static final int countChars(CharSequence charSequence, int n, int n2) {
        if (n2 == 1 && !Character.isHighSurrogate(charSequence.charAt(n))) {
            assert n >= 0 && n < charSequence.length();
            return 1;
        }
        else {
            int length = charSequence.length();
            if (n2 >= 0) {
                assert n >= 0 && n < length;
                for (int n5 = 0; n < length && n5 < n2; ++n5) {
                    if (Character.isHighSurrogate(charSequence.charAt(n++)) && n < length && Character.isLowSurrogate(charSequence.charAt(n))) {
                        ++n;
                    }
                }
                return n - n;
            }
            else {
                assert n >= 0 && n <= length;
                if (n == 0) {
                    return 0;
                }
                for (int n3 = -n2, n4 = 0; n > 0 && n4 < n3; ++n4) {
                    if (Character.isLowSurrogate(charSequence.charAt(--n)) && n > 0 && Character.isHighSurrogate(charSequence.charAt(n - 1))) {
                        --n;
                    }
                }
                return n - n;
            }
        }
    }
    
    private static final int countCodePoints(CharSequence charSequence) {
        int length = charSequence.length();
        int n = 0;
        int i = 0;
        while (i < length) {
            ++n;
            if (!Character.isHighSurrogate(charSequence.charAt(i++)) || i >= length || !Character.isLowSurrogate(charSequence.charAt(i))) {
                continue;
            }
            ++i;
        }
        return n;
    }
    
    private Pattern.CharProperty newSingle(int codePoint) {
        if (this.has(2)) {
            if (this.has(64)) {
                int upperCase = Character.toUpperCase(codePoint);
                int lowerCase = Character.toLowerCase(upperCase);
                if (upperCase != lowerCase) {
                    return new SingleU(lowerCase);
                }
            }
            else if (ASCII.isAscii(codePoint)) {
                int lower = ASCII.toLower(codePoint);
                int upper = ASCII.toUpper(codePoint);
                if (lower != upper) {
                    return new SingleI(lower, upper);
                }
            }
        }
        if (isSupplementary(codePoint)) {
            return new SingleS(codePoint);
        }
        return new Single(codePoint);
    }
    
    private Pattern.Node newSlice(int[] array, int n, boolean b) {
        int[] array2 = new int[n];
        if (this.has(2)) {
            if (this.has(64)) {
                for (int k = 0; k < n; ++k) {
                    array2[k] = Character.toLowerCase(Character.toUpperCase(array[k]));
                }
                return b ? new SliceUS(array2) : new SliceU(array2);
            }
            for (int j = 0; j < n; ++j) {
                array2[j] = ASCII.toLower(array[j]);
            }
            return b ? new SliceIS(array2) : new SliceI(array2);
        }
        else {
            for (int i = 0; i < n; ++i) {
                array2[i] = array[i];
            }
            return b ? new SliceS(array2) : new Slice(array2);
        }
    }
    
    private static boolean inRange(int n, int n2, int n3) {
        return n <= n2 && n2 <= n3;
    }
    
    private static Pattern.CharProperty rangeFor(final int n, final int n2) {
        return new Pattern.CharProperty() {
            @Override
            boolean isSatisfiedBy(int n) {
                return Pattern.access$200(n, n, n2);
            }
        };
    }
    
    private Pattern.CharProperty caseInsensitiveRangeFor(final int n, final int n2) {
        if (this.has(64)) {
            return new Pattern.CharProperty() {
                @Override
                boolean isSatisfiedBy(int codePoint) {
                    if (Pattern.access$200(n, codePoint, n2)) {
                        return true;
                    }
                    int upperCase = Character.toUpperCase(codePoint);
                    return Pattern.access$200(n, upperCase, n2) || Pattern.access$200(n, Character.toLowerCase(upperCase), n2);
                }
            };
        }
        return new Pattern.CharProperty() {
            @Override
            boolean isSatisfiedBy(int n) {
                return Pattern.access$200(n, n, n2) || (ASCII.isAscii(n) && (Pattern.access$200(n, ASCII.toUpper(n), n2) || Pattern.access$200(n, ASCII.toLower(n), n2)));
            }
        };
    }
    
    private static Pattern.CharProperty union(final Pattern.CharProperty charProperty, final Pattern.CharProperty charProperty2) {
        return new Pattern.CharProperty() {
            @Override
            boolean isSatisfiedBy(int n) {
                return charProperty.isSatisfiedBy(n) || charProperty2.isSatisfiedBy(n);
            }
        };
    }
    
    private static Pattern.CharProperty intersection(final Pattern.CharProperty charProperty, final Pattern.CharProperty charProperty2) {
        return new Pattern.CharProperty() {
            @Override
            boolean isSatisfiedBy(int n) {
                return charProperty.isSatisfiedBy(n) && charProperty2.isSatisfiedBy(n);
            }
        };
    }
    
    private static Pattern.CharProperty setDifference(final Pattern.CharProperty charProperty, final Pattern.CharProperty charProperty2) {
        return new Pattern.CharProperty() {
            @Override
            boolean isSatisfiedBy(int n) {
                return !charProperty2.isSatisfiedBy(n) && charProperty.isSatisfiedBy(n);
            }
        };
    }
    
    private static boolean hasBaseCharacter(Matcher matcher, int i, CharSequence seq) {
        int n = matcher.transparentBounds ? 0 : matcher.from;
        while (i >= n) {
            int codePoint = Character.codePointAt(seq, i);
            if (Character.isLetterOrDigit(codePoint)) {
                return true;
            }
            if (Character.getType(codePoint) == 6) {
                --i;
            }
            else {
                return false;
            }
        }
        return false;
    }
    
    public Predicate<String> asPredicate() {
        return s/* java.lang.String, */ -> this.matcher(s).find();
    }
    
    public Stream<String> splitAsStream(CharSequence charSequence) {
        class MatcherIterator implements Iterator<String>
        {
            private final Matcher matcher;
            private int current;
            private String nextElement;
            private int emptyElementCount;
            
            MatcherIterator() {
                this.matcher = Pattern.this.matcher(charSequence);
            }
            
            @Override
            public String next() {
                if (!this.hasNext()) {
                    throw new NoSuchElementException();
                }
                if (this.emptyElementCount == 0) {
                    String nextElement = this.nextElement;
                    this.nextElement = null;
                    return nextElement;
                }
                --this.emptyElementCount;
                return "";
            }
            
            @Override
            public boolean hasNext() {
                if (this.nextElement != null || this.emptyElementCount > 0) {
                    return true;
                }
                if (this.current == charSequence.length()) {
                    return false;
                }
                while (this.matcher.find()) {
                    this.nextElement = charSequence.subSequence(this.current, this.matcher.start()).toString();
                    this.current = this.matcher.end();
                    if (!this.nextElement.isEmpty()) {
                        return true;
                    }
                    if (this.current <= 0) {
                        continue;
                    }
                    ++this.emptyElementCount;
                }
                this.nextElement = charSequence.subSequence(this.current, charSequence.length()).toString();
                this.current = charSequence.length();
                if (!this.nextElement.isEmpty()) {
                    return true;
                }
                this.emptyElementCount = 0;
                this.nextElement = null;
                return false;
            }
        }
        return StreamSupport.stream(Spliterators.spliteratorUnknownSize(new MatcherIterator(), 272), false);
    }
    
    static boolean access$200(int n, int n2, int n3) {
        return inRange(n, n2, n3);
    }
    
    static int access$300(CharSequence charSequence, int n, int n2) {
        return countChars(charSequence, n, n2);
    }
    
    static boolean access$400(Matcher matcher, int n, CharSequence charSequence) {
        return hasBaseCharacter(matcher, n, charSequence);
    }
    
    static Pattern.CharProperty access$600(int n, int n2) {
        return rangeFor(n, n2);
    }
    
    static {
        Pattern.lookbehindEnd = new Pattern.Node() {
            @Override
            boolean match(Matcher matcher, int n, CharSequence charSequence) {
                return n == matcher.lookbehindTo;
            }
        };
        Pattern.accept = new Pattern.Node();
        Pattern.lastAccept = new Pattern.LastNode();
    }
    
    static final class TreeInfo
    {
        int minLength;
        int maxLength;
        boolean maxValid;
        boolean deterministic;
        
        TreeInfo() {
            this.reset();
        }
        
        void reset() {
            this.minLength = 0;
            this.maxLength = 0;
            this.maxValid = true;
            this.deterministic = true;
        }
    }
    
    private static final class BitClass extends Pattern.BmpCharProperty
    {
        final boolean[] bits;
        
        BitClass() {
            this.bits = new boolean[256];
        }
        
        private BitClass(boolean[] bits) {
            this.bits = bits;
        }
        
        Pattern.BitClass add(int n, int n2) {
            assert n >= 0 && n <= 255;
            if ((n2 & 0x2) != 0x0) {
                if (ASCII.isAscii(n)) {
                    this.bits[ASCII.toUpper(n)] = true;
                    this.bits[ASCII.toLower(n)] = true;
                }
                else if ((n2 & 0x40) != 0x0) {
                    this.bits[Character.toLowerCase(n)] = true;
                    this.bits[Character.toUpperCase(n)] = true;
                }
            }
            this.bits[n] = true;
            return this;
        }
        
        @Override
        boolean isSatisfiedBy(int n) {
            return n < 256 && this.bits[n];
        }
    }
    
    static class Node
    {
        Pattern.Node next;
        
        Node() {
            this.next = Pattern.accept;
        }
        
        boolean match(Matcher matcher, int last, CharSequence charSequence) {
            matcher.last = last;
            matcher.groups[0] = matcher.first;
            matcher.groups[1] = matcher.last;
            return true;
        }
        
        boolean study(Pattern.TreeInfo treeInfo) {
            if (this.next != null) {
                return this.next.study(treeInfo);
            }
            return treeInfo.deterministic;
        }
    }
    
    static class LastNode extends Pattern.Node
    {
        @Override
        boolean match(Matcher matcher, int last, CharSequence charSequence) {
            if (matcher.acceptMode == 1 && last != matcher.to) {
                return false;
            }
            matcher.last = last;
            matcher.groups[0] = matcher.first;
            matcher.groups[1] = matcher.last;
            return true;
        }
    }
    
    static class Start extends Pattern.Node
    {
        int minLength;
        
        Start(Pattern.Node next) {
            this.next = next;
            Pattern.TreeInfo treeInfo = new Pattern.TreeInfo();
            this.next.study(treeInfo);
            this.minLength = treeInfo.minLength;
        }
        
        @Override
        boolean match(Matcher matcher, int i, CharSequence charSequence) {
            if (i > matcher.to - this.minLength) {
                matcher.hitEnd = true;
                return false;
            }
            while (i <= matcher.to - this.minLength) {
                if (this.next.match(matcher, i, charSequence)) {
                    matcher.first = i;
                    matcher.groups[0] = matcher.first;
                    matcher.groups[1] = matcher.last;
                    return true;
                }
                ++i;
            }
            matcher.hitEnd = true;
            return false;
        }
        
        @Override
        boolean study(Pattern.TreeInfo treeInfo) {
            this.next.study(treeInfo);
            treeInfo.maxValid = false;
            return treeInfo.deterministic = false;
        }
    }
    
    static final class StartS extends Pattern.Start
    {
        StartS(Pattern.Node node) {
            super(node);
        }
        
        @Override
        boolean match(Matcher matcher, int i, CharSequence charSequence) {
            if (i > matcher.to - this.minLength) {
                matcher.hitEnd = true;
                return false;
            }
            int n = matcher.to - this.minLength;
            while (i <= n) {
                if (this.next.match(matcher, i, charSequence)) {
                    matcher.first = i;
                    matcher.groups[0] = matcher.first;
                    matcher.groups[1] = matcher.last;
                    return true;
                }
                if (i == n) {
                    break;
                }
                if (!Character.isHighSurrogate(charSequence.charAt(i++)) || i >= charSequence.length() || !Character.isLowSurrogate(charSequence.charAt(i))) {
                    continue;
                }
                ++i;
            }
            matcher.hitEnd = true;
            return false;
        }
    }
    
    static final class Begin extends Pattern.Node
    {
        @Override
        boolean match(Matcher matcher, int first, CharSequence charSequence) {
            if (first == (matcher.anchoringBounds ? matcher.from : 0) && this.next.match(matcher, first, charSequence)) {
                matcher.first = first;
                matcher.groups[0] = first;
                matcher.groups[1] = matcher.last;
                return true;
            }
            return false;
        }
    }
    
    static final class End extends Pattern.Node
    {
        @Override
        boolean match(Matcher matcher, int n, CharSequence charSequence) {
            if (n == (matcher.anchoringBounds ? matcher.to : matcher.getTextLength())) {
                matcher.hitEnd = true;
                return this.next.match(matcher, n, charSequence);
            }
            return false;
        }
    }
    
    static final class Caret extends Pattern.Node
    {
        @Override
        boolean match(Matcher matcher, int n, CharSequence charSequence) {
            int from = matcher.from;
            int n2 = matcher.to;
            if (!matcher.anchoringBounds) {
                from = 0;
                n2 = matcher.getTextLength();
            }
            if (n == n2) {
                matcher.hitEnd = true;
                return false;
            }
            if (n > from) {
                int char1 = charSequence.charAt(n - 1);
                if (char1 != '\n' && char1 != '\r' && (char1 | '\u0001') != 0x2029 && char1 != '\u0085') {
                    return false;
                }
                if (char1 == '\r' && charSequence.charAt(n) == '\n') {
                    return false;
                }
            }
            return this.next.match(matcher, n, charSequence);
        }
    }
    
    static final class UnixCaret extends Pattern.Node
    {
        @Override
        boolean match(Matcher matcher, int n, CharSequence charSequence) {
            int from = matcher.from;
            int n2 = matcher.to;
            if (!matcher.anchoringBounds) {
                from = 0;
                n2 = matcher.getTextLength();
            }
            if (n == n2) {
                matcher.hitEnd = true;
                return false;
            }
            return (n <= from || charSequence.charAt(n - 1) == '\n') && this.next.match(matcher, n, charSequence);
        }
    }
    
    static final class LastMatch extends Pattern.Node
    {
        @Override
        boolean match(Matcher matcher, int n, CharSequence charSequence) {
            return n == matcher.oldLast && this.next.match(matcher, n, charSequence);
        }
    }
    
    static final class Dollar extends Pattern.Node
    {
        boolean multiline;
        
        Dollar(boolean multiline) {
            this.multiline = multiline;
        }
        
        @Override
        boolean match(Matcher matcher, int n, CharSequence charSequence) {
            int n2 = matcher.anchoringBounds ? matcher.to : matcher.getTextLength();
            if (!this.multiline) {
                if (n < n2 - 2) {
                    return false;
                }
                if (n == n2 - 2) {
                    if (charSequence.charAt(n) != '\r') {
                        return false;
                    }
                    if (charSequence.charAt(n + 1) != '\n') {
                        return false;
                    }
                }
            }
            if (n < n2) {
                int char1 = charSequence.charAt(n);
                if (char1 == '\n') {
                    if (n > 0 && charSequence.charAt(n - 1) == '\r') {
                        return false;
                    }
                    if (this.multiline) {
                        return this.next.match(matcher, n, charSequence);
                    }
                }
                else if (char1 == '\r' || char1 == '\u0085' || (char1 | '\u0001') == 0x2029) {
                    if (this.multiline) {
                        return this.next.match(matcher, n, charSequence);
                    }
                }
                else {
                    return false;
                }
            }
            matcher.hitEnd = true;
            matcher.requireEnd = true;
            return this.next.match(matcher, n, charSequence);
        }
        
        @Override
        boolean study(Pattern.TreeInfo treeInfo) {
            this.next.study(treeInfo);
            return treeInfo.deterministic;
        }
    }
    
    static final class UnixDollar extends Pattern.Node
    {
        boolean multiline;
        
        UnixDollar(boolean multiline) {
            this.multiline = multiline;
        }
        
        @Override
        boolean match(Matcher matcher, int n, CharSequence charSequence) {
            int n2 = matcher.anchoringBounds ? matcher.to : matcher.getTextLength();
            if (n < n2) {
                if (charSequence.charAt(n) == '\n') {
                    if (!this.multiline && n != n2 - 1) {
                        return false;
                    }
                    if (this.multiline) {
                        return this.next.match(matcher, n, charSequence);
                    }
                }
                else {
                    return false;
                }
            }
            matcher.hitEnd = true;
            matcher.requireEnd = true;
            return this.next.match(matcher, n, charSequence);
        }
        
        @Override
        boolean study(Pattern.TreeInfo treeInfo) {
            this.next.study(treeInfo);
            return treeInfo.deterministic;
        }
    }
    
    static final class LineEnding extends Pattern.Node
    {
        @Override
        boolean match(Matcher matcher, int n, CharSequence charSequence) {
            if (n < matcher.to) {
                int char1 = charSequence.charAt(n);
                if (char1 == '\n' || char1 == '\u000b' || char1 == '\f' || char1 == '\u0085' || char1 == '\u2028' || char1 == '\u2029') {
                    return this.next.match(matcher, n + 1, charSequence);
                }
                if (char1 == '\r') {
                    if (++n < matcher.to && charSequence.charAt(n) == '\n') {
                        ++n;
                    }
                    return this.next.match(matcher, n, charSequence);
                }
            }
            else {
                matcher.hitEnd = true;
            }
            return false;
        }
        
        @Override
        boolean study(Pattern.TreeInfo treeInfo) {
            ++treeInfo.minLength;
            treeInfo.maxLength += 2;
            return this.next.study(treeInfo);
        }
    }
    
    private abstract static class CharProperty extends Pattern.Node
    {
        abstract boolean isSatisfiedBy(int p0);
        
        Pattern.CharProperty complement() {
            return new Pattern.CharProperty() {
                @Override
                boolean isSatisfiedBy(int n) {
                    return !super.isSatisfiedBy(n);
                }
            };
        }
        
        @Override
        boolean match(Matcher matcher, int index, CharSequence seq) {
            if (index < matcher.to) {
                int codePoint = Character.codePointAt(seq, index);
                return this.isSatisfiedBy(codePoint) && this.next.match(matcher, index + Character.charCount(codePoint), seq);
            }
            matcher.hitEnd = true;
            return false;
        }
        
        @Override
        boolean study(Pattern.TreeInfo treeInfo) {
            ++treeInfo.minLength;
            ++treeInfo.maxLength;
            return this.next.study(treeInfo);
        }
    }
    
    private abstract static class BmpCharProperty extends Pattern.CharProperty
    {
        @Override
        boolean match(Matcher matcher, int n, CharSequence charSequence) {
            if (n < matcher.to) {
                return this.isSatisfiedBy((int)charSequence.charAt(n)) && this.next.match(matcher, n + 1, charSequence);
            }
            matcher.hitEnd = true;
            return false;
        }
    }
    
    static final class SingleS extends Pattern.CharProperty
    {
        final int c;
        
        SingleS(int c) {
            this.c = c;
        }
        
        @Override
        boolean isSatisfiedBy(int n) {
            return n == this.c;
        }
    }
    
    static final class Single extends Pattern.BmpCharProperty
    {
        final int c;
        
        Single(int c) {
            this.c = c;
        }
        
        @Override
        boolean isSatisfiedBy(int n) {
            return n == this.c;
        }
    }
    
    static final class SingleI extends Pattern.BmpCharProperty
    {
        final int lower;
        final int upper;
        
        SingleI(int lower, int upper) {
            this.lower = lower;
            this.upper = upper;
        }
        
        @Override
        boolean isSatisfiedBy(int n) {
            return n == this.lower || n == this.upper;
        }
    }
    
    static final class SingleU extends Pattern.CharProperty
    {
        final int lower;
        
        SingleU(int lower) {
            this.lower = lower;
        }
        
        @Override
        boolean isSatisfiedBy(int codePoint) {
            return this.lower == codePoint || this.lower == Character.toLowerCase(Character.toUpperCase(codePoint));
        }
    }
    
    static final class Block extends Pattern.CharProperty
    {
        final Character.UnicodeBlock block;
        
        Block(Character.UnicodeBlock block) {
            this.block = block;
        }
        
        @Override
        boolean isSatisfiedBy(int codePoint) {
            return this.block == Character.UnicodeBlock.of(codePoint);
        }
    }
    
    static final class Script extends Pattern.CharProperty
    {
        final Character.UnicodeScript script;
        
        Script(Character.UnicodeScript script) {
            this.script = script;
        }
        
        @Override
        boolean isSatisfiedBy(int codePoint) {
            return this.script == Character.UnicodeScript.of(codePoint);
        }
    }
    
    static final class Category extends Pattern.CharProperty
    {
        final int typeMask;
        
        Category(int typeMask) {
            this.typeMask = typeMask;
        }
        
        @Override
        boolean isSatisfiedBy(int codePoint) {
            return (this.typeMask & 1 << Character.getType(codePoint)) != 0x0;
        }
    }
    
    static final class Utype extends Pattern.CharProperty
    {
        final UnicodeProp uprop;
        
        Utype(UnicodeProp uprop) {
            this.uprop = uprop;
        }
        
        @Override
        boolean isSatisfiedBy(int n) {
            return this.uprop.is(n);
        }
    }
    
    static final class Ctype extends Pattern.BmpCharProperty
    {
        final int ctype;
        
        Ctype(int ctype) {
            this.ctype = ctype;
        }
        
        @Override
        boolean isSatisfiedBy(int n) {
            return n < 128 && ASCII.isType(n, this.ctype);
        }
    }
    
    static final class VertWS extends Pattern.BmpCharProperty
    {
        @Override
        boolean isSatisfiedBy(int n) {
            return (n >= 10 && n <= 13) || n == 133 || n == 8232 || n == 8233;
        }
    }
    
    static final class HorizWS extends Pattern.BmpCharProperty
    {
        @Override
        boolean isSatisfiedBy(int n) {
            return n == 9 || n == 32 || n == 160 || n == 5760 || n == 6158 || (n >= 8192 && n <= 8202) || n == 8239 || n == 8287 || n == 12288;
        }
    }
    
    static class SliceNode extends Pattern.Node
    {
        int[] buffer;
        
        SliceNode(int[] buffer) {
            this.buffer = buffer;
        }
        
        @Override
        boolean study(Pattern.TreeInfo treeInfo) {
            treeInfo.minLength += this.buffer.length;
            treeInfo.maxLength += this.buffer.length;
            return this.next.study(treeInfo);
        }
    }
    
    static final class Slice extends Pattern.SliceNode
    {
        Slice(int[] array) {
            super(array);
        }
        
        @Override
        boolean match(Matcher matcher, int n, CharSequence charSequence) {
            int[] buffer = this.buffer;
            int length = buffer.length;
            int i = 0;
            while (i < length) {
                if (n + i >= matcher.to) {
                    matcher.hitEnd = true;
                    return false;
                }
                if (buffer[i] != charSequence.charAt(n + i)) {
                    return false;
                }
                ++i;
            }
            return this.next.match(matcher, n + length, charSequence);
        }
    }
    
    static class SliceI extends Pattern.SliceNode
    {
        SliceI(int[] array) {
            super(array);
        }
        
        @Override
        boolean match(Matcher matcher, int n, CharSequence charSequence) {
            int[] buffer = this.buffer;
            int length = buffer.length;
            int i = 0;
            while (i < length) {
                if (n + i >= matcher.to) {
                    matcher.hitEnd = true;
                    return false;
                }
                int char1 = charSequence.charAt(n + i);
                if (buffer[i] != char1 && buffer[i] != ASCII.toLower(char1)) {
                    return false;
                }
                ++i;
            }
            return this.next.match(matcher, n + length, charSequence);
        }
    }
    
    static final class SliceU extends Pattern.SliceNode
    {
        SliceU(int[] array) {
            super(array);
        }
        
        @Override
        boolean match(Matcher matcher, int n, CharSequence charSequence) {
            int[] buffer = this.buffer;
            int length = buffer.length;
            int i = 0;
            while (i < length) {
                if (n + i >= matcher.to) {
                    matcher.hitEnd = true;
                    return false;
                }
                int char1 = charSequence.charAt(n + i);
                if (buffer[i] != char1 && buffer[i] != Character.toLowerCase(Character.toUpperCase(char1))) {
                    return false;
                }
                ++i;
            }
            return this.next.match(matcher, n + length, charSequence);
        }
    }
    
    static final class SliceS extends Pattern.SliceNode
    {
        SliceS(int[] array) {
            super(array);
        }
        
        @Override
        boolean match(Matcher matcher, int index, CharSequence seq) {
            int[] buffer = this.buffer;
            int i = 0;
            while (i < buffer.length) {
                if (index >= matcher.to) {
                    matcher.hitEnd = true;
                    return false;
                }
                int codePoint = Character.codePointAt(seq, index);
                if (buffer[i] != codePoint) {
                    return false;
                }
                index += Character.charCount(codePoint);
                if (index > matcher.to) {
                    matcher.hitEnd = true;
                    return false;
                }
                ++i;
            }
            return this.next.match(matcher, index, seq);
        }
    }
    
    static class SliceIS extends Pattern.SliceNode
    {
        SliceIS(int[] array) {
            super(array);
        }
        
        int toLower(int n) {
            return ASCII.toLower(n);
        }
        
        @Override
        boolean match(Matcher matcher, int index, CharSequence seq) {
            int[] buffer = this.buffer;
            int i = 0;
            while (i < buffer.length) {
                if (index >= matcher.to) {
                    matcher.hitEnd = true;
                    return false;
                }
                int codePoint = Character.codePointAt(seq, index);
                if (buffer[i] != codePoint && buffer[i] != this.toLower(codePoint)) {
                    return false;
                }
                index += Character.charCount(codePoint);
                if (index > matcher.to) {
                    matcher.hitEnd = true;
                    return false;
                }
                ++i;
            }
            return this.next.match(matcher, index, seq);
        }
    }
    
    static final class SliceUS extends Pattern.SliceIS
    {
        SliceUS(int[] array) {
            super(array);
        }
        
        @Override
        int toLower(int codePoint) {
            return Character.toLowerCase(Character.toUpperCase(codePoint));
        }
    }
    
    static final class All extends Pattern.CharProperty
    {
        @Override
        boolean isSatisfiedBy(int n) {
            return true;
        }
    }
    
    static final class Dot extends Pattern.CharProperty
    {
        @Override
        boolean isSatisfiedBy(int n) {
            return n != 10 && n != 13 && (n | 0x1) != 0x2029 && n != 133;
        }
    }
    
    static final class UnixDot extends Pattern.CharProperty
    {
        @Override
        boolean isSatisfiedBy(int n) {
            return n != 10;
        }
    }
    
    static final class Ques extends Pattern.Node
    {
        Pattern.Node atom;
        int type;
        
        Ques(Pattern.Node atom, int type) {
            this.atom = atom;
            this.type = type;
        }
        
        @Override
        boolean match(Matcher matcher, int last, CharSequence charSequence) {
            switch (this.type) {
                case 0: {
                    return (this.atom.match(matcher, last, charSequence) && this.next.match(matcher, matcher.last, charSequence)) || this.next.match(matcher, last, charSequence);
                }
                case 1: {
                    return this.next.match(matcher, last, charSequence) || (this.atom.match(matcher, last, charSequence) && this.next.match(matcher, matcher.last, charSequence));
                }
                case 2: {
                    if (this.atom.match(matcher, last, charSequence)) {
                        last = matcher.last;
                    }
                    return this.next.match(matcher, last, charSequence);
                }
                default: {
                    return this.atom.match(matcher, last, charSequence) && this.next.match(matcher, matcher.last, charSequence);
                }
            }
        }
        
        @Override
        boolean study(Pattern.TreeInfo treeInfo) {
            if (this.type != 3) {
                int minLength = treeInfo.minLength;
                this.atom.study(treeInfo);
                treeInfo.minLength = minLength;
                treeInfo.deterministic = false;
                return this.next.study(treeInfo);
            }
            this.atom.study(treeInfo);
            return this.next.study(treeInfo);
        }
    }
    
    static final class Curly extends Pattern.Node
    {
        Pattern.Node atom;
        int type;
        int cmin;
        int cmax;
        
        Curly(Pattern.Node atom, int cmin, int cmax, int type) {
            this.atom = atom;
            this.type = type;
            this.cmin = cmin;
            this.cmax = cmax;
        }
        
        @Override
        boolean match(Matcher matcher, int last, CharSequence charSequence) {
            int i = 0;
            while (i < this.cmin) {
                if (this.atom.match(matcher, last, charSequence)) {
                    last = matcher.last;
                    ++i;
                }
                else {
                    return false;
                }
            }
            if (this.type == 0) {
                return this.match0(matcher, last, i, charSequence);
            }
            if (this.type == 1) {
                return this.match1(matcher, last, i, charSequence);
            }
            return this.match2(matcher, last, i, charSequence);
        }
        
        boolean match0(Matcher matcher, int last, int i, CharSequence charSequence) {
            if (i >= this.cmax) {
                return this.next.match(matcher, last, charSequence);
            }
            if (this.atom.match(matcher, last, charSequence)) {
                int n = matcher.last - last;
                if (n != 0) {
                    last = matcher.last;
                    ++i;
                    while (i < this.cmax) {
                        if (!this.atom.match(matcher, last, charSequence)) {
                            break;
                        }
                        if (last + n != matcher.last) {
                            if (this.match0(matcher, matcher.last, i + 1, charSequence)) {
                                return true;
                            }
                            break;
                        }
                        else {
                            last += n;
                            ++i;
                        }
                    }
                    while (i >= i) {
                        if (this.next.match(matcher, last, charSequence)) {
                            return true;
                        }
                        last -= n;
                        --i;
                    }
                    return false;
                }
            }
            return this.next.match(matcher, last, charSequence);
        }
        
        boolean match1(Matcher matcher, int last, int n, CharSequence charSequence) {
            while (!this.next.match(matcher, last, charSequence)) {
                if (n >= this.cmax) {
                    return false;
                }
                if (!this.atom.match(matcher, last, charSequence)) {
                    return false;
                }
                if (last == matcher.last) {
                    return false;
                }
                last = matcher.last;
                ++n;
            }
            return true;
        }
        
        boolean match2(Matcher matcher, int last, int i, CharSequence charSequence) {
            while (i < this.cmax) {
                if (!this.atom.match(matcher, last, charSequence)) {
                    break;
                }
                if (last == matcher.last) {
                    break;
                }
                last = matcher.last;
                ++i;
            }
            return this.next.match(matcher, last, charSequence);
        }
        
        @Override
        boolean study(Pattern.TreeInfo treeInfo) {
            int minLength = treeInfo.minLength;
            int maxLength = treeInfo.maxLength;
            int maxValid = treeInfo.maxValid;
            int deterministic = treeInfo.deterministic;
            treeInfo.reset();
            this.atom.study(treeInfo);
            int minLength2 = treeInfo.minLength * this.cmin + minLength;
            if (minLength2 < minLength) {
                minLength2 = 268435455;
            }
            treeInfo.minLength = minLength2;
            if (!((maxValid & (treeInfo.maxValid ? 1 : 0)) == 0x0)) {
                if ((treeInfo.maxLength = treeInfo.maxLength * this.cmax + maxLength) < maxLength) {
                    treeInfo.maxValid = false;
                }
            }
            else {
                treeInfo.maxValid = false;
            }
            if (treeInfo.deterministic && this.cmin == this.cmax) {
                treeInfo.deterministic = deterministic;
            }
            else {
                treeInfo.deterministic = false;
            }
            return this.next.study(treeInfo);
        }
    }
    
    static final class GroupCurly extends Pattern.Node
    {
        Pattern.Node atom;
        int type;
        int cmin;
        int cmax;
        int localIndex;
        int groupIndex;
        boolean capture;
        
        GroupCurly(Pattern.Node atom, int cmin, int cmax, int type, int localIndex, int groupIndex, boolean capture) {
            this.atom = atom;
            this.type = type;
            this.cmin = cmin;
            this.cmax = cmax;
            this.localIndex = localIndex;
            this.groupIndex = groupIndex;
            this.capture = capture;
        }
        
        @Override
        boolean match(Matcher matcher, int last, CharSequence charSequence) {
            int[] groups = matcher.groups;
            int[] locals = matcher.locals;
            int n = locals[this.localIndex];
            int n2 = 0;
            int n3 = 0;
            if (this.capture) {
                n2 = groups[this.groupIndex];
                n3 = groups[this.groupIndex + 1];
            }
            locals[this.localIndex] = -1;
            boolean n4 = true;
            int i = 0;
            while (i < this.cmin) {
                if (this.atom.match(matcher, last, charSequence)) {
                    if (this.capture) {
                        groups[this.groupIndex] = last;
                        groups[this.groupIndex + 1] = matcher.last;
                    }
                    last = matcher.last;
                    ++i;
                }
                else {
                    n4 = false;
                    break;
                }
            }
            if (!(n4 == 0)) {
                if (this.type == 0) {
                    n4 = this.match0(matcher, last, this.cmin, charSequence);
                }
                else if (this.type == 1) {
                    n4 = this.match1(matcher, last, this.cmin, charSequence);
                }
                else {
                    n4 = this.match2(matcher, last, this.cmin, charSequence);
                }
            }
            if (n4 == 0) {
                locals[this.localIndex] = n;
                if (this.capture) {
                    groups[this.groupIndex] = n2;
                    groups[this.groupIndex + 1] = n3;
                }
            }
            return n4;
        }
        
        boolean match0(Matcher matcher, int n, int i, CharSequence charSequence) {
            int[] groups = matcher.groups;
            int n2 = 0;
            int n3 = 0;
            if (this.capture) {
                n2 = groups[this.groupIndex];
                n3 = groups[this.groupIndex + 1];
            }
            Label_0297: {
                if (i < this.cmax && this.atom.match(matcher, n, charSequence)) {
                    int n4 = matcher.last - n;
                    if (n4 <= 0) {
                        if (this.capture) {
                            groups[this.groupIndex] = n;
                            groups[this.groupIndex + 1] = n + n4;
                        }
                        n += n4;
                    }
                    else {
                        Label_0208: {
                            do {
                                if (this.capture) {
                                    groups[this.groupIndex] = n;
                                    groups[this.groupIndex + 1] = n + n4;
                                }
                                n += n4;
                                if (++i < this.cmax && this.atom.match(matcher, n, charSequence)) {
                                    continue;
                                }
                                while (i > i) {
                                    if (this.next.match(matcher, n, charSequence)) {
                                        if (this.capture) {
                                            groups[this.groupIndex + 1] = n;
                                            groups[this.groupIndex] = n - n4;
                                        }
                                        return true;
                                    }
                                    else {
                                        n -= n4;
                                        if (this.capture) {
                                            groups[this.groupIndex + 1] = n;
                                            groups[this.groupIndex] = n - n4;
                                        }
                                        --i;
                                    }
                                }
                                break Label_0297;
                            } while (n + n4 == matcher.last);
                            if (this.match0(matcher, n, i, charSequence)) {
                                return true;
                            }
                            continue;
                        }
                    }
                }
            }
            if (this.capture) {
                groups[this.groupIndex] = n2;
                groups[this.groupIndex + 1] = n3;
            }
            return this.next.match(matcher, n, charSequence);
        }
        
        boolean match1(Matcher matcher, int last, int n, CharSequence charSequence) {
            while (!this.next.match(matcher, last, charSequence)) {
                if (n >= this.cmax) {
                    return false;
                }
                if (!this.atom.match(matcher, last, charSequence)) {
                    return false;
                }
                if (last == matcher.last) {
                    return false;
                }
                if (this.capture) {
                    matcher.groups[this.groupIndex] = last;
                    matcher.groups[this.groupIndex + 1] = matcher.last;
                }
                last = matcher.last;
                ++n;
            }
            return true;
        }
        
        boolean match2(Matcher matcher, int last, int i, CharSequence charSequence) {
            while (i < this.cmax) {
                if (!this.atom.match(matcher, last, charSequence)) {
                    break;
                }
                if (this.capture) {
                    matcher.groups[this.groupIndex] = last;
                    matcher.groups[this.groupIndex + 1] = matcher.last;
                }
                if (last == matcher.last) {
                    break;
                }
                last = matcher.last;
                ++i;
            }
            return this.next.match(matcher, last, charSequence);
        }
        
        @Override
        boolean study(Pattern.TreeInfo treeInfo) {
            int minLength = treeInfo.minLength;
            int maxLength = treeInfo.maxLength;
            int maxValid = treeInfo.maxValid;
            int deterministic = treeInfo.deterministic;
            treeInfo.reset();
            this.atom.study(treeInfo);
            int minLength2 = treeInfo.minLength * this.cmin + minLength;
            if (minLength2 < minLength) {
                minLength2 = 268435455;
            }
            treeInfo.minLength = minLength2;
            if (!((maxValid & (treeInfo.maxValid ? 1 : 0)) == 0x0)) {
                if ((treeInfo.maxLength = treeInfo.maxLength * this.cmax + maxLength) < maxLength) {
                    treeInfo.maxValid = false;
                }
            }
            else {
                treeInfo.maxValid = false;
            }
            if (treeInfo.deterministic && this.cmin == this.cmax) {
                treeInfo.deterministic = deterministic;
            }
            else {
                treeInfo.deterministic = false;
            }
            return this.next.study(treeInfo);
        }
    }
    
    static final class BranchConn extends Pattern.Node
    {
        @Override
        boolean match(Matcher matcher, int n, CharSequence charSequence) {
            return this.next.match(matcher, n, charSequence);
        }
        
        @Override
        boolean study(Pattern.TreeInfo treeInfo) {
            return treeInfo.deterministic;
        }
    }
    
    static final class Branch extends Pattern.Node
    {
        Pattern.Node[] atoms;
        int size;
        Pattern.Node conn;
        
        Branch(Pattern.Node node, Pattern.Node node2, Pattern.Node conn) {
            this.atoms = new Pattern.Node[2];
            this.size = 2;
            this.conn = conn;
            this.atoms[0] = node;
            this.atoms[1] = node2;
        }
        
        void add(Pattern.Node node) {
            if (this.size >= this.atoms.length) {
                Pattern.Node[] atoms = new Pattern.Node[this.atoms.length * 2];
                System.arraycopy(this.atoms, 0, atoms, 0, this.atoms.length);
                this.atoms = atoms;
            }
            this.atoms[this.size++] = node;
        }
        
        @Override
        boolean match(Matcher matcher, int n, CharSequence charSequence) {
            for (int i = 0; i < this.size; ++i) {
                if (this.atoms[i] == null) {
                    if (this.conn.next.match(matcher, n, charSequence)) {
                        return true;
                    }
                }
                else if (this.atoms[i].match(matcher, n, charSequence)) {
                    return true;
                }
            }
            return false;
        }
        
        @Override
        boolean study(Pattern.TreeInfo treeInfo) {
            int minLength = treeInfo.minLength;
            int maxLength = treeInfo.maxLength;
            int maxValid = treeInfo.maxValid;
            int min = Integer.MAX_VALUE;
            int max = -1;
            for (int i = 0; i < this.size; min = Math.min(min, treeInfo.minLength), max = Math.max(max, treeInfo.maxLength), maxValid &= treeInfo.maxValid, ++i) {
                treeInfo.reset();
                if (this.atoms[i] != null) {
                    this.atoms[i].study(treeInfo);
                }
            }
            int n = minLength + min;
            int n2 = maxLength + max;
            treeInfo.reset();
            this.conn.next.study(treeInfo);
            treeInfo.minLength += n;
            treeInfo.maxLength += n2;
            treeInfo.maxValid &= maxValid;
            return treeInfo.deterministic = false;
        }
    }
    
    static final class GroupHead extends Pattern.Node
    {
        int localIndex;
        
        GroupHead(int localIndex) {
            this.localIndex = localIndex;
        }
        
        @Override
        boolean match(Matcher matcher, int n, CharSequence charSequence) {
            int n2 = matcher.locals[this.localIndex];
            matcher.locals[this.localIndex] = n;
            int match = this.next.match(matcher, n, charSequence);
            matcher.locals[this.localIndex] = n2;
            return match;
        }
        
        boolean matchRef(Matcher matcher, int n, CharSequence charSequence) {
            int n2 = matcher.locals[this.localIndex];
            matcher.locals[this.localIndex] = ~n;
            int match = this.next.match(matcher, n, charSequence);
            matcher.locals[this.localIndex] = n2;
            return match;
        }
    }
    
    static final class GroupRef extends Pattern.Node
    {
        Pattern.GroupHead head;
        
        GroupRef(Pattern.GroupHead head) {
            this.head = head;
        }
        
        @Override
        boolean match(Matcher matcher, int n, CharSequence charSequence) {
            return this.head.matchRef(matcher, n, charSequence) && this.next.match(matcher, matcher.last, charSequence);
        }
        
        @Override
        boolean study(Pattern.TreeInfo treeInfo) {
            treeInfo.maxValid = false;
            treeInfo.deterministic = false;
            return this.next.study(treeInfo);
        }
    }
    
    static final class GroupTail extends Pattern.Node
    {
        int localIndex;
        int groupIndex;
        
        GroupTail(int localIndex, int n) {
            this.localIndex = localIndex;
            this.groupIndex = n + n;
        }
        
        @Override
        boolean match(Matcher matcher, int last, CharSequence charSequence) {
            int n = matcher.locals[this.localIndex];
            if (n >= 0) {
                int n2 = matcher.groups[this.groupIndex];
                int n3 = matcher.groups[this.groupIndex + 1];
                matcher.groups[this.groupIndex] = n;
                matcher.groups[this.groupIndex + 1] = last;
                if (this.next.match(matcher, last, charSequence)) {
                    return true;
                }
                matcher.groups[this.groupIndex] = n2;
                matcher.groups[this.groupIndex + 1] = n3;
                return false;
            }
            else {
                matcher.last = last;
                return true;
            }
        }
    }
    
    static final class Prolog extends Pattern.Node
    {
        Pattern.Loop loop;
        
        Prolog(Pattern.Loop loop) {
            this.loop = loop;
        }
        
        @Override
        boolean match(Matcher matcher, int n, CharSequence charSequence) {
            return this.loop.matchInit(matcher, n, charSequence);
        }
        
        @Override
        boolean study(Pattern.TreeInfo treeInfo) {
            return this.loop.study(treeInfo);
        }
    }
    
    static class Loop extends Pattern.Node
    {
        Pattern.Node body;
        int countIndex;
        int beginIndex;
        int cmin;
        int cmax;
        
        Loop(int countIndex, int beginIndex) {
            this.countIndex = countIndex;
            this.beginIndex = beginIndex;
        }
        
        @Override
        boolean match(Matcher matcher, int n, CharSequence charSequence) {
            if (n > matcher.locals[this.beginIndex]) {
                int n2 = matcher.locals[this.countIndex];
                if (n2 < this.cmin) {
                    matcher.locals[this.countIndex] = n2 + 1;
                    int match = this.body.match(matcher, n, charSequence);
                    if (match == 0) {
                        matcher.locals[this.countIndex] = n2;
                    }
                    return match;
                }
                else if (n2 < this.cmax) {
                    matcher.locals[this.countIndex] = n2 + 1;
                    if (!this.body.match(matcher, n, charSequence)) {
                        matcher.locals[this.countIndex] = n2;
                    }
                    else {
                        return true;
                    }
                }
            }
            return this.next.match(matcher, n, charSequence);
        }
        
        boolean matchInit(Matcher matcher, int n, CharSequence charSequence) {
            int n2 = matcher.locals[this.countIndex];
            int n3;
            if (0 < this.cmin) {
                matcher.locals[this.countIndex] = 1;
                n3 = this.body.match(matcher, n, charSequence);
            }
            else if (0 < this.cmax) {
                matcher.locals[this.countIndex] = 1;
                n3 = this.body.match(matcher, n, charSequence);
                if (n3 == 0) {
                    n3 = this.next.match(matcher, n, charSequence);
                }
            }
            else {
                n3 = this.next.match(matcher, n, charSequence);
            }
            matcher.locals[this.countIndex] = n2;
            return n3;
        }
        
        @Override
        boolean study(Pattern.TreeInfo treeInfo) {
            treeInfo.maxValid = false;
            return treeInfo.deterministic = false;
        }
    }
    
    static final class LazyLoop extends Pattern.Loop
    {
        LazyLoop(int n, int n2) {
            super(n, n2);
        }
        
        @Override
        boolean match(Matcher matcher, int n, CharSequence charSequence) {
            if (n > matcher.locals[this.beginIndex]) {
                int n2 = matcher.locals[this.countIndex];
                if (n2 < this.cmin) {
                    matcher.locals[this.countIndex] = n2 + 1;
                    int match2 = this.body.match(matcher, n, charSequence);
                    if (match2 == 0) {
                        matcher.locals[this.countIndex] = n2;
                    }
                    return match2;
                }
                else {
                    if (this.next.match(matcher, n, charSequence)) {
                        return true;
                    }
                    if (n2 < this.cmax) {
                        matcher.locals[this.countIndex] = n2 + 1;
                        int match = this.body.match(matcher, n, charSequence);
                        if (match == 0) {
                            matcher.locals[this.countIndex] = n2;
                        }
                        return match;
                    }
                    else {
                        return false;
                    }
                }
            }
            else {
                return this.next.match(matcher, n, charSequence);
            }
        }
        
        @Override
        boolean matchInit(Matcher matcher, int n, CharSequence charSequence) {
            int n2 = matcher.locals[this.countIndex];
            boolean n3 = false;
            if (0 < this.cmin) {
                matcher.locals[this.countIndex] = 1;
                n3 = this.body.match(matcher, n, charSequence);
            }
            else if (this.next.match(matcher, n, charSequence)) {
                n3 = true;
            }
            else if (0 < this.cmax) {
                matcher.locals[this.countIndex] = 1;
                n3 = this.body.match(matcher, n, charSequence);
            }
            matcher.locals[this.countIndex] = n2;
            return n3;
        }
        
        @Override
        boolean study(Pattern.TreeInfo treeInfo) {
            treeInfo.maxValid = false;
            return treeInfo.deterministic = false;
        }
    }
    
    static class BackRef extends Pattern.Node
    {
        int groupIndex;
        
        BackRef(int n) {
            this.groupIndex = n + n;
        }
        
        @Override
        boolean match(Matcher matcher, int n, CharSequence charSequence) {
            int n2 = matcher.groups[this.groupIndex];
            int n3 = matcher.groups[this.groupIndex + 1] - n2;
            if (n2 < 0) {
                return false;
            }
            if (n + n3 > matcher.to) {
                matcher.hitEnd = true;
                return false;
            }
            int i = 0;
            while (i < n3) {
                if (charSequence.charAt(n + i) != charSequence.charAt(n2 + i)) {
                    return false;
                }
                ++i;
            }
            return this.next.match(matcher, n + n3, charSequence);
        }
        
        @Override
        boolean study(Pattern.TreeInfo treeInfo) {
            treeInfo.maxValid = false;
            return this.next.study(treeInfo);
        }
    }
    
    static class CIBackRef extends Pattern.Node
    {
        int groupIndex;
        boolean doUnicodeCase;
        
        CIBackRef(int n, boolean doUnicodeCase) {
            this.groupIndex = n + n;
            this.doUnicodeCase = doUnicodeCase;
        }
        
        @Override
        boolean match(Matcher matcher, int index, CharSequence charSequence) {
            int index2 = matcher.groups[this.groupIndex];
            int n = matcher.groups[this.groupIndex + 1] - index2;
            if (index2 < 0) {
                return false;
            }
            if (index + n > matcher.to) {
                matcher.hitEnd = true;
                return false;
            }
            int codePoint;
            int codePoint2;
            for (int i = 0; i < n; index += Character.charCount(codePoint), index2 += Character.charCount(codePoint2), ++i) {
                codePoint = Character.codePointAt(charSequence, index);
                codePoint2 = Character.codePointAt(charSequence, index2);
                if (codePoint != codePoint2) {
                    if (this.doUnicodeCase) {
                        int upperCase = Character.toUpperCase(codePoint);
                        int upperCase2 = Character.toUpperCase(codePoint2);
                        if (upperCase != upperCase2 && Character.toLowerCase(upperCase) != Character.toLowerCase(upperCase2)) {
                            return false;
                        }
                    }
                    else if (ASCII.toLower(codePoint) != ASCII.toLower(codePoint2)) {
                        return false;
                    }
                }
            }
            return this.next.match(matcher, index + n, charSequence);
        }
        
        @Override
        boolean study(Pattern.TreeInfo treeInfo) {
            treeInfo.maxValid = false;
            return this.next.study(treeInfo);
        }
    }
    
    static final class First extends Pattern.Node
    {
        Pattern.Node atom;
        
        First(Pattern.Node node) {
            this.atom = Pattern.BnM.optimize(node);
        }
        
        @Override
        boolean match(Matcher matcher, int i, CharSequence charSequence) {
            if ((this.atom instanceof Pattern.BnM)) {
                return this.atom.match(matcher, i, charSequence) && this.next.match(matcher, matcher.last, charSequence);
            }
            while (i <= matcher.to) {
                if (this.atom.match(matcher, i, charSequence)) {
                    return this.next.match(matcher, matcher.last, charSequence);
                }
                i += Pattern.access$300(charSequence, i, 1);
                ++matcher.first;
            }
            matcher.hitEnd = true;
            return false;
        }
        
        @Override
        boolean study(Pattern.TreeInfo treeInfo) {
            this.atom.study(treeInfo);
            treeInfo.maxValid = false;
            treeInfo.deterministic = false;
            return this.next.study(treeInfo);
        }
    }
    
    static final class Conditional extends Pattern.Node
    {
        Pattern.Node cond;
        Pattern.Node yes;
        Pattern.Node not;
        
        Conditional(Pattern.Node cond, Pattern.Node yes, Pattern.Node not) {
            this.cond = cond;
            this.yes = yes;
            this.not = not;
        }
        
        @Override
        boolean match(Matcher matcher, int n, CharSequence charSequence) {
            if (this.cond.match(matcher, n, charSequence)) {
                return this.yes.match(matcher, n, charSequence);
            }
            return this.not.match(matcher, n, charSequence);
        }
        
        @Override
        boolean study(Pattern.TreeInfo treeInfo) {
            int minLength = treeInfo.minLength;
            int maxLength = treeInfo.maxLength;
            int maxValid = treeInfo.maxValid;
            treeInfo.reset();
            this.yes.study(treeInfo);
            int minLength2 = treeInfo.minLength;
            int maxLength2 = treeInfo.maxLength;
            int maxValid2 = treeInfo.maxValid;
            treeInfo.reset();
            this.not.study(treeInfo);
            treeInfo.minLength = minLength + Math.min(minLength2, treeInfo.minLength);
            treeInfo.maxLength = maxLength + Math.max(maxLength2, treeInfo.maxLength);
            treeInfo.maxValid &= ((maxValid & maxValid2) != 0x0);
            treeInfo.deterministic = false;
            return this.next.study(treeInfo);
        }
    }
    
    static final class Pos extends Pattern.Node
    {
        Pattern.Node cond;
        
        Pos(Pattern.Node cond) {
            this.cond = cond;
        }
        
        @Override
        boolean match(Matcher matcher, int n, CharSequence charSequence) {
            int to = matcher.to;
            boolean match = false;
            if (matcher.transparentBounds) {
                matcher.to = matcher.getTextLength();
            }
            try {
                match = this.cond.match(matcher, n, charSequence);
            }
            finally {
                matcher.to = to;
            }
            return match && this.next.match(matcher, n, charSequence);
        }
    }
    
    static final class Neg extends Pattern.Node
    {
        Pattern.Node cond;
        
        Neg(Pattern.Node cond) {
            this.cond = cond;
        }
        
        @Override
        boolean match(Matcher matcher, int n, CharSequence charSequence) {
            int to = matcher.to;
            boolean n2 = false;
            if (matcher.transparentBounds) {
                matcher.to = matcher.getTextLength();
            }
            try {
                if (n < matcher.to) {
                    n2 = !this.cond.match(matcher, n, charSequence);
                }
                else {
                    matcher.requireEnd = true;
                    n2 = !this.cond.match(matcher, n, charSequence);
                }
            }
            finally {
                matcher.to = to;
            }
            return n2 && this.next.match(matcher, n, charSequence);
        }
    }
    
    static class Behind extends Pattern.Node
    {
        Pattern.Node cond;
        int rmax;
        int rmin;
        
        Behind(Pattern.Node cond, int rmax, int rmin) {
            this.cond = cond;
            this.rmax = rmax;
            this.rmin = rmin;
        }
        
        @Override
        boolean match(Matcher matcher, int lookbehindTo, CharSequence charSequence) {
            int from = matcher.from;
            boolean match = false;
            int max = Math.max(lookbehindTo - this.rmax, matcher.transparentBounds ? 0 : matcher.from);
            int lookbehindTo2 = matcher.lookbehindTo;
            matcher.lookbehindTo = lookbehindTo;
            if (matcher.transparentBounds) {
                matcher.from = 0;
            }
            for (int n = lookbehindTo - this.rmin; match == 0 && n >= max; match = this.cond.match(matcher, n, charSequence), --n) {}
            matcher.from = from;
            matcher.lookbehindTo = lookbehindTo2;
            return match && this.next.match(matcher, lookbehindTo, charSequence);
        }
    }
    
    static final class BehindS extends Pattern.Behind
    {
        BehindS(Pattern.Node node, int n, int n2) {
            super(node, n, n2);
        }
        
        @Override
        boolean match(Matcher matcher, int lookbehindTo, CharSequence charSequence) {
            int access$300 = Pattern.access$300(charSequence, lookbehindTo, -this.rmax);
            int access$301 = Pattern.access$300(charSequence, lookbehindTo, -this.rmin);
            int from = matcher.from;
            int b = matcher.transparentBounds ? 0 : matcher.from;
            boolean match = false;
            int max = Math.max(lookbehindTo - access$300, b);
            int lookbehindTo2 = matcher.lookbehindTo;
            matcher.lookbehindTo = lookbehindTo;
            if (matcher.transparentBounds) {
                matcher.from = 0;
            }
            for (int n = lookbehindTo - access$301; match == 0 && n >= max; match = this.cond.match(matcher, n, charSequence), n -= ((n > max) ? Pattern.access$300(charSequence, n, -1) : 1)) {}
            matcher.from = from;
            matcher.lookbehindTo = lookbehindTo2;
            return match && this.next.match(matcher, lookbehindTo, charSequence);
        }
    }
    
    static class NotBehind extends Pattern.Node
    {
        Pattern.Node cond;
        int rmax;
        int rmin;
        
        NotBehind(Pattern.Node cond, int rmax, int rmin) {
            this.cond = cond;
            this.rmax = rmax;
            this.rmin = rmin;
        }
        
        @Override
        boolean match(Matcher matcher, int lookbehindTo, CharSequence charSequence) {
            int lookbehindTo2 = matcher.lookbehindTo;
            int from = matcher.from;
            boolean match = false;
            int max = Math.max(lookbehindTo - this.rmax, matcher.transparentBounds ? 0 : matcher.from);
            matcher.lookbehindTo = lookbehindTo;
            if (matcher.transparentBounds) {
                matcher.from = 0;
            }
            for (int n = lookbehindTo - this.rmin; match == 0 && n >= max; match = this.cond.match(matcher, n, charSequence), --n) {}
            matcher.from = from;
            matcher.lookbehindTo = lookbehindTo2;
            return match == 0 && this.next.match(matcher, lookbehindTo, charSequence);
        }
    }
    
    static final class NotBehindS extends Pattern.NotBehind
    {
        NotBehindS(Pattern.Node node, int n, int n2) {
            super(node, n, n2);
        }
        
        @Override
        boolean match(Matcher matcher, int lookbehindTo, CharSequence charSequence) {
            int access$300 = Pattern.access$300(charSequence, lookbehindTo, -this.rmax);
            int access$301 = Pattern.access$300(charSequence, lookbehindTo, -this.rmin);
            int from = matcher.from;
            int lookbehindTo2 = matcher.lookbehindTo;
            boolean match = false;
            int max = Math.max(lookbehindTo - access$300, matcher.transparentBounds ? 0 : matcher.from);
            matcher.lookbehindTo = lookbehindTo;
            if (matcher.transparentBounds) {
                matcher.from = 0;
            }
            for (int n = lookbehindTo - access$301; match == 0 && n >= max; match = this.cond.match(matcher, n, charSequence), n -= ((n > max) ? Pattern.access$300(charSequence, n, -1) : 1)) {}
            matcher.from = from;
            matcher.lookbehindTo = lookbehindTo2;
            return match == 0 && this.next.match(matcher, lookbehindTo, charSequence);
        }
    }
    
    static final class Bound extends Pattern.Node
    {
        static int LEFT;
        static int RIGHT;
        static int BOTH;
        static int NONE;
        int type;
        boolean useUWORD;
        
        Bound(int type, boolean useUWORD) {
            this.type = type;
            this.useUWORD = useUWORD;
        }
        
        boolean isWord(int codePoint) {
            return this.useUWORD ? UnicodeProp.WORD.is(codePoint) : (codePoint == 95 || Character.isLetterOrDigit(codePoint));
        }
        
        int check(Matcher matcher, int n, CharSequence charSequence) {
            boolean n2 = false;
            int from = matcher.from;
            int n3 = matcher.to;
            if (matcher.transparentBounds) {
                from = 0;
                n3 = matcher.getTextLength();
            }
            if (n > from) {
                int codePointBefore = Character.codePointBefore(charSequence, n);
                n2 = (this.isWord(codePointBefore) || (Character.getType(codePointBefore) == 6 && Pattern.access$400(matcher, n - 1, charSequence)));
            }
            boolean n4 = false;
            if (n < n3) {
                int codePoint = Character.codePointAt(charSequence, n);
                n4 = (this.isWord(codePoint) || (Character.getType(codePoint) == 6 && Pattern.access$400(matcher, n, charSequence)));
            }
            else {
                matcher.hitEnd = true;
                matcher.requireEnd = true;
            }
            return ((n2 ^ n4) != 0x0) ? (n4 ? Pattern.Bound.LEFT : Pattern.Bound.RIGHT) : Pattern.Bound.NONE;
        }
        
        @Override
        boolean match(Matcher matcher, int n, CharSequence charSequence) {
            return (this.check(matcher, n, charSequence) & this.type) > 0 && this.next.match(matcher, n, charSequence);
        }
        
        static {
            Pattern.Bound.LEFT = 1;
            Pattern.Bound.RIGHT = 2;
            Pattern.Bound.BOTH = 3;
            Pattern.Bound.NONE = 4;
        }
    }
    
    static class BnM extends Pattern.Node
    {
        int[] buffer;
        int[] lastOcc;
        int[] optoSft;
        
        static Pattern.Node optimize(Pattern.Node node) {
            if (!(node instanceof Pattern.Slice)) {
                return node;
            }
            int[] buffer = ((Pattern.Slice)node).buffer;
            int i = buffer.length;
            if (i < 4) {
                return node;
            }
            int[] array = new int[128];
            int[] array2 = new int[i];
            for (int j = 0; j < i; ++j) {
                array[buffer[j] & 0x7F] = j + 1;
            }
        Label_0066:
            while (i > 0) {
                int k = i - 1;
                Label_0125: {
                    while (k >= i) {
                        if (buffer[k] == buffer[k - i]) {
                            array2[k - 1] = i;
                            --k;
                        }
                        else {
                            --i;
                            continue Label_0066;
                        }
                    }
                    while (k > 0) {
                        array2[--k] = i;
                    }
                    continue;
                }
            }
            array2[i - 1] = 1;
            if ((node instanceof Pattern.SliceS)) {
                return new Pattern.BnMS(buffer, array, array2, node.next);
            }
            return new Pattern.BnM(buffer, array, array2, node.next);
        }
        
        BnM(int[] buffer, int[] lastOcc, int[] optoSft, Pattern.Node next) {
            this.buffer = buffer;
            this.lastOcc = lastOcc;
            this.optoSft = optoSft;
            this.next = next;
        }
        
        @Override
        boolean match(Matcher matcher, int i, CharSequence charSequence) {
            int[] buffer = this.buffer;
            int length = buffer.length;
            int n = matcher.to - length;
        Label_0020:
            while (i <= n) {
                int j = length - 1;
                while (j >= 0) {
                    int char1 = charSequence.charAt(i + j);
                    if (char1 != buffer[j]) {
                        i += Math.max(j + 1 - this.lastOcc[char1 & '\u007f'], this.optoSft[j]);
                        continue Label_0020;
                    }
                    --j;
                }
                matcher.first = i;
                if (this.next.match(matcher, i + length, charSequence)) {
                    matcher.first = i;
                    matcher.groups[0] = matcher.first;
                    matcher.groups[1] = matcher.last;
                    return true;
                }
                ++i;
            }
            matcher.hitEnd = true;
            return false;
        }
        
        @Override
        boolean study(Pattern.TreeInfo treeInfo) {
            treeInfo.minLength += this.buffer.length;
            treeInfo.maxValid = false;
            return this.next.study(treeInfo);
        }
    }
    
    static final class BnMS extends Pattern.BnM
    {
        int lengthInChars;
        
        BnMS(int[] array, int[] array2, int[] array3, Pattern.Node node) {
            super(array, array2, array3, node);
            for (int i = 0; i < this.buffer.length; ++i) {
                this.lengthInChars += Character.charCount(this.buffer[i]);
            }
        }
        
        @Override
        boolean match(Matcher matcher, int i, CharSequence seq) {
            int[] buffer = this.buffer;
            int length = buffer.length;
            int n = matcher.to - this.lengthInChars;
        Label_0022:
            while (i <= n) {
                int j = Pattern.access$300(seq, i, length);
                int n2 = length - 1;
                while (j > 0) {
                    int codePointBefore = Character.codePointBefore(seq, i + j);
                    if (codePointBefore != buffer[n2]) {
                        i += Pattern.access$300(seq, i, Math.max(n2 + 1 - this.lastOcc[codePointBefore & 0x7F], this.optoSft[n2]));
                        continue Label_0022;
                    }
                    j -= Character.charCount(codePointBefore);
                    --n2;
                }
                matcher.first = i;
                if (this.next.match(matcher, i + this.lengthInChars, seq)) {
                    matcher.first = i;
                    matcher.groups[0] = matcher.first;
                    matcher.groups[1] = matcher.last;
                    return true;
                }
                i += Pattern.access$300(seq, i, 1);
            }
            matcher.hitEnd = true;
            return false;
        }
    }
    
    private static class CharPropertyNames
    {
        private static final HashMap<String, Pattern.CharPropertyNames.CharPropertyFactory> map;
        
        static Pattern.CharProperty charPropertyFor(String key) {
            Pattern.CharPropertyNames.CharPropertyFactory charPropertyFactory = (Pattern.CharPropertyNames.CharPropertyFactory)Pattern.CharPropertyNames.map.get(key);
            return (charPropertyFactory == null) ? null : charPropertyFactory.make();
        }
        
        private static void defCategory(String key, int n) {
            Pattern.CharPropertyNames.map.put(key, new Pattern.CharPropertyNames.CharPropertyFactory() {
                @Override
                Pattern.CharProperty make() {
                    return new Category(n);
                }
            });
        }
        
        private static void defRange(String key, int n, int n2) {
            Pattern.CharPropertyNames.map.put(key, new Pattern.CharPropertyNames.CharPropertyFactory() {
                @Override
                Pattern.CharProperty make() {
                    return Pattern.access$600(n, n2);
                }
            });
        }
        
        private static void defCtype(String key, int n) {
            Pattern.CharPropertyNames.map.put(key, new Pattern.CharPropertyNames.CharPropertyFactory() {
                @Override
                Pattern.CharProperty make() {
                    return new Ctype(n);
                }
            });
        }
        
        private static void defClone(String key, Pattern.CharPropertyNames.CloneableProperty cloneableProperty) {
            Pattern.CharPropertyNames.map.put(key, new Pattern.CharPropertyNames.CharPropertyFactory() {
                @Override
                Pattern.CharProperty make() {
                    return cloneableProperty.clone();
                }
            });
        }
        
        static {
            map = new HashMap();
            defCategory("Cn", 1);
            defCategory("Lu", 2);
            defCategory("Ll", 4);
            defCategory("Lt", 8);
            defCategory("Lm", 16);
            defCategory("Lo", 32);
            defCategory("Mn", 64);
            defCategory("Me", 128);
            defCategory("Mc", 256);
            defCategory("Nd", 512);
            defCategory("Nl", 1024);
            defCategory("No", 2048);
            defCategory("Zs", 4096);
            defCategory("Zl", 8192);
            defCategory("Zp", 16384);
            defCategory("Cc", 32768);
            defCategory("Cf", 65536);
            defCategory("Co", 262144);
            defCategory("Cs", 524288);
            defCategory("Pd", 1048576);
            defCategory("Ps", 2097152);
            defCategory("Pe", 4194304);
            defCategory("Pc", 8388608);
            defCategory("Po", 16777216);
            defCategory("Sm", 33554432);
            defCategory("Sc", 67108864);
            defCategory("Sk", 134217728);
            defCategory("So", 268435456);
            defCategory("Pi", 536870912);
            defCategory("Pf", 1073741824);
            defCategory("L", 62);
            defCategory("M", 448);
            defCategory("N", 3584);
            defCategory("Z", 28672);
            defCategory("C", 884736);
            defCategory("P", 1643118592);
            defCategory("S", 503316480);
            defCategory("LC", 14);
            defCategory("LD", 574);
            defRange("L1", 0, 255);
            Pattern.CharPropertyNames.map.put("all", new Pattern.CharPropertyNames.CharPropertyFactory() {
                @Override
                Pattern.CharProperty make() {
                    return new Pattern.All();
                }
            });
            defRange("ASCII", 0, 127);
            defCtype("Alnum", 1792);
            defCtype("Alpha", 768);
            defCtype("Blank", 16384);
            defCtype("Cntrl", 8192);
            defRange("Digit", 48, 57);
            defCtype("Graph", 5888);
            defRange("Lower", 97, 122);
            defRange("Print", 32, 126);
            defCtype("Punct", 4096);
            defCtype("Space", 2048);
            defRange("Upper", 65, 90);
            defCtype("XDigit", 32768);
            defClone("javaLowerCase", new Pattern.CharPropertyNames.CloneableProperty() {
                @Override
                boolean isSatisfiedBy(int codePoint) {
                    return Character.isLowerCase(codePoint);
                }
            });
            defClone("javaUpperCase", new Pattern.CharPropertyNames.CloneableProperty() {
                @Override
                boolean isSatisfiedBy(int codePoint) {
                    return Character.isUpperCase(codePoint);
                }
            });
            defClone("javaAlphabetic", new Pattern.CharPropertyNames.CloneableProperty() {
                @Override
                boolean isSatisfiedBy(int codePoint) {
                    return Character.isAlphabetic(codePoint);
                }
            });
            defClone("javaIdeographic", new Pattern.CharPropertyNames.CloneableProperty() {
                @Override
                boolean isSatisfiedBy(int codePoint) {
                    return Character.isIdeographic(codePoint);
                }
            });
            defClone("javaTitleCase", new Pattern.CharPropertyNames.CloneableProperty() {
                @Override
                boolean isSatisfiedBy(int codePoint) {
                    return Character.isTitleCase(codePoint);
                }
            });
            defClone("javaDigit", new Pattern.CharPropertyNames.CloneableProperty() {
                @Override
                boolean isSatisfiedBy(int codePoint) {
                    return Character.isDigit(codePoint);
                }
            });
            defClone("javaDefined", new Pattern.CharPropertyNames.CloneableProperty() {
                @Override
                boolean isSatisfiedBy(int codePoint) {
                    return Character.isDefined(codePoint);
                }
            });
            defClone("javaLetter", new Pattern.CharPropertyNames.CloneableProperty() {
                @Override
                boolean isSatisfiedBy(int codePoint) {
                    return Character.isLetter(codePoint);
                }
            });
            defClone("javaLetterOrDigit", new Pattern.CharPropertyNames.CloneableProperty() {
                @Override
                boolean isSatisfiedBy(int codePoint) {
                    return Character.isLetterOrDigit(codePoint);
                }
            });
            defClone("javaJavaIdentifierStart", new Pattern.CharPropertyNames.CloneableProperty() {
                @Override
                boolean isSatisfiedBy(int codePoint) {
                    return Character.isJavaIdentifierStart(codePoint);
                }
            });
            defClone("javaJavaIdentifierPart", new Pattern.CharPropertyNames.CloneableProperty() {
                @Override
                boolean isSatisfiedBy(int codePoint) {
                    return Character.isJavaIdentifierPart(codePoint);
                }
            });
            defClone("javaUnicodeIdentifierStart", new Pattern.CharPropertyNames.CloneableProperty() {
                @Override
                boolean isSatisfiedBy(int codePoint) {
                    return Character.isUnicodeIdentifierStart(codePoint);
                }
            });
            defClone("javaUnicodeIdentifierPart", new Pattern.CharPropertyNames.CloneableProperty() {
                @Override
                boolean isSatisfiedBy(int codePoint) {
                    return Character.isUnicodeIdentifierPart(codePoint);
                }
            });
            defClone("javaIdentifierIgnorable", new Pattern.CharPropertyNames.CloneableProperty() {
                @Override
                boolean isSatisfiedBy(int codePoint) {
                    return Character.isIdentifierIgnorable(codePoint);
                }
            });
            defClone("javaSpaceChar", new Pattern.CharPropertyNames.CloneableProperty() {
                @Override
                boolean isSatisfiedBy(int codePoint) {
                    return Character.isSpaceChar(codePoint);
                }
            });
            defClone("javaWhitespace", new Pattern.CharPropertyNames.CloneableProperty() {
                @Override
                boolean isSatisfiedBy(int codePoint) {
                    return Character.isWhitespace(codePoint);
                }
            });
            defClone("javaISOControl", new Pattern.CharPropertyNames.CloneableProperty() {
                @Override
                boolean isSatisfiedBy(int codePoint) {
                    return Character.isISOControl(codePoint);
                }
            });
            defClone("javaMirrored", new Pattern.CharPropertyNames.CloneableProperty() {
                @Override
                boolean isSatisfiedBy(int codePoint) {
                    return Character.isMirrored(codePoint);
                }
            });
        }
        
        private abstract static class CharPropertyFactory
        {
            abstract Pattern.CharProperty make();
        }
        
        private abstract static class CloneableProperty extends Pattern.CharProperty implements Cloneable
        {
            public Pattern.CharPropertyNames.CloneableProperty clone() {
                try {
                    return (Pattern.CharPropertyNames.CloneableProperty)super.clone();
                }
                catch (CloneNotSupportedException detailMessage) {
                    throw new AssertionError(detailMessage);
                }
            }
        }
    }
}
